function [data, APD] = HAMM(settings, paramname, outputname)
	if nargin < 3 || isempty(outputname), outputname = 'Output_HAMM_Model'; end
	if nargin < 2 || isempty(paramname), paramname = 'Parameters_HAMM.txt'; end
	
	settings = setDefaultSettings(settings);
 
	fid = fopen([paramname],'w');    
		fprintf(fid,'bcl = %f\n', settings.bcl);
		fprintf(fid,'ft = %f\n', settings.ft);
		fprintf(fid,'startsave = %f\n', settings.startsave);
		fprintf(fid,'Output_APD = %s\n', settings.Output_APD);
		fprintf(fid,'levelOfElectroDetail = %d\n', settings.levelOfElectroDetail);
		fprintf(fid,'Adaptive_dt_Alpha = %f\n', settings.Adaptive_dt_Alpha);
		if isfield(settings, 'DT_MIN'), fprintf(fid,'DT_MIN = %f\n', settings.DT_MIN); end
		if isfield(settings, 'DT_MAX'), fprintf(fid,'DT_MAX = %f\n', settings.DT_MAX); end
		if isfield(settings, 'skip'), fprintf(fid,'skip = %f\n', settings.skip); end
		if isfield(settings, 'update'), fprintf(fid,'update = %d\n', settings.update); end
		if isfield(settings, 'cellType'), fprintf(fid,'cellType = %d\n', settings.cellType); end
		if isfield(settings, 'Istim'), fprintf(fid,'Istim = %d\n', settings.Istim); end
		if isfield(settings, 'stimdur'), fprintf(fid,'stimdur = %d\n', settings.stimdur); end
		if isfield(settings, 'applyVoltageClamp'), fprintf(fid, 'applyVoltageClamp = %s\n', settings.applyVoltageClamp); end
		if isfield(settings, 'VC_Values'), fprintf(fid, 'VC_Values = %s\n', (sprintf("%g,",settings.VC_Values))(1:end-1)); end
		if isfield(settings, 'VC_Times'), fprintf(fid, 'VC_Times = %s\n', (sprintf("%g,",settings.VC_Times))(1:end-1)); end
		if isfield(settings, 'Flec'), fprintf(fid, 'Flec = %f\n', settings.Flec); end
		if isfield(settings, 'IK1B'), fprintf(fid, 'IK1B = %f\n', settings.IK1B); end
		if isfield(settings, 'IKrB'), fprintf(fid, 'IKrB = %f\n', settings.IKrB); end
		if isfield(settings, 'IKsB'), fprintf(fid, 'IKsB = %f\n', settings.IKsB); end
		if isfield(settings, 'IKurB'), fprintf(fid, 'IKurB = %f\n', settings.IKurB); end
		if isfield(settings, 'IKpB'), fprintf(fid, 'IKpB = %f\n', settings.IKpB); end
		if isfield(settings, 'ITo1B'), fprintf(fid, 'ITo1B = %f\n', settings.ITo1B); end
		if isfield(settings, 'INaB'), fprintf(fid, 'INaB = %f\n', settings.INaB); end
		if isfield(settings, 'IrelB'), fprintf(fid, 'IrelB = %f\n', settings.IrelB); end
		if isfield(settings, 'IleakB'), fprintf(fid, 'IleakB = %f\n', settings.IleakB); end
		if isfield(settings, 'IupB'), fprintf(fid, 'IupB = %f\n', settings.IupB); end
		if isfield(settings, 'INaLB'), fprintf(fid, 'INaLB = %f\n', settings.INaLB); end
		if isfield(settings, 'INaKB'), fprintf(fid, 'INaKB = %f\n', settings.INaKB); end
		if isfield(settings, 'INaCaB'), fprintf(fid, 'INaCaB = %f\n', settings.INaCaB); end
		if isfield(settings, 'ICaLB'), fprintf(fid, 'ICaLB = %f\n', settings.ICaLB); end
		if isfield(settings, 'IClCaB'), fprintf(fid, 'ITo2B = %f\n', settings.IClCaB); end
		if isfield(settings, 'IClbB'), fprintf(fid, 'IClbB = %f\n', settings.IClbB); end
		if isfield(settings, 'ICabB'), fprintf(fid, 'ICabB = %f\n', settings.ICabB); end
		if isfield(settings, 'IpCaB'), fprintf(fid, 'IpCaB = %f\n', settings.IpCaB); end
		if isfield(settings, 'Na_o'), fprintf(fid, 'Na_o = %f\n', settings.Na_o); end
		if isfield(settings, 'Ca_o'), fprintf(fid, 'Ca_o = %f\n', settings.Ca_o); end
		if isfield(settings, 'K_o'), fprintf(fid, 'K_o = %f\n', settings.K_o); end
		if isfield(settings, 'EGTA'), fprintf(fid, 'EGTA = %f\n', settings.EGTA); end
		if isfield(settings, 'Fluo3'), fprintf(fid, 'Fluo3 = %f\n', settings.Fluo3); end
		if isfield(settings, 'Clamp_K_i'), fprintf(fid, 'Clamp_K_i = %f\n', settings.Clamp_K_i); end
		if isfield(settings, 'Clamp_Na_i'), fprintf(fid, 'Clamp_Na_i = %f\n', settings.Clamp_Na_i); end
		if isfield(settings, 'simIK1NaDependence'), fprintf(fid, 'simIK1NaDependence = %s\n', settings.simIK1NaDependence); end
		if isfield(settings, 'simIKAch'), fprintf(fid, 'simIKAch = %s\n', settings.simIKAch); end
		if isfield(settings, 'simIKAchNaDep'), fprintf(fid, 'simIKAChNaDep = %d\n', settings.simIKAchNaDep); end
		if isfield(settings, 'useCourtemancheINa'), fprintf(fid, 'useCourtemancheINa = %s\n', settings.useCourtemancheINa); end
		if isfield(settings, 'hideOutput'), fprintf(fid, 'hideOutput = %s\n', settings.hideOutput); end
		if isfield(settings, 'N_Segments'), fprintf(fid, 'N_Segments = %d\n', settings.N_Segments); end
		if isfield(settings, 'N_CaDomains'), fprintf(fid, 'N_CaDomains = %d\n', settings.N_CaDomains); end
		if isfield(settings, 'segOutputRes'), fprintf(fid, 'segOutputRes = %d\n', settings.segOutputRes); end
		if isfield(settings, 'domOutputRes'), fprintf(fid, 'domOutputRes = %d\n', settings.domOutputRes); end
		if isfield(settings, 'RyRModelType'), fprintf(fid, 'RyRModelType = %d\n', settings.RyRModelType); end		
		if isfield(settings, 'allowLongitudinalSRSDiff'), fprintf(fid, 'allowLongitudinalSRSDiff = %s\n', settings.allowLongitudinalSRSDiff); end		
		if isfield(settings, 'RyR_NP_Params'), fprintf(fid, 'RyR_NP_Params = %s\n', (sprintf("%g,",settings.RyR_NP_Params))(1:end-1)); end
		if isfield(settings, 'Params_ICaL'), fprintf(fid, 'Params_ICaL = %s\n', (sprintf("%g,",settings.Params_ICaL))(1:end-1)); end
		if isfield(settings, 'tau_diff_Casr_Domain'), fprintf(fid, 'tau_diff_Casr_Domain = %g\n', settings.tau_diff_Casr_Domain); end
		if isfield(settings, 'tau_diff_Casr_Segment'), fprintf(fid, 'tau_diff_Casr_Segment = %g\n', settings.tau_diff_Casr_Segment); end
		if isfield(settings, 'tau_diff_Cai_Domain'), fprintf(fid, 'tau_diff_Cai_Domain = %g\n', settings.tau_diff_Cai_Domain); end
		if isfield(settings, 'tau_diff_Cai_Segment'), fprintf(fid, 'tau_diff_Cai_Segment = %g\n', settings.tau_diff_Cai_Segment); end
		if isfield(settings, 'tau_diff_Casrs_Domain'), fprintf(fid, 'tau_diff_Casrs_Domain = %g\n', settings.tau_diff_Casrs_Domain); end
		if isfield(settings, 'tau_diff_Casrs_Segment'), fprintf(fid, 'tau_diff_Casrs_Segment = %g\n', settings.tau_diff_Casrs_Segment); end
		if isfield(settings, 'tau_diff_SRS_i'), fprintf(fid, 'tau_diff_SRS_i = %g\n', settings.tau_diff_SRS_i); end
		if isfield(settings, 'factDiffJuncSl'), fprintf(fid, 'factDiffJuncSl = %g\n', settings.factDiffJuncSl); end
		if isfield(settings, 'factDiffSlMyo'), fprintf(fid, 'factDiffSlMyo = %g\n', settings.factDiffSlMyo); end
		if isfield(settings, 'factBmaxCSQN'), fprintf(fid, 'factBmaxCSQN = %g\n', settings.factBmaxCSQN); end
		if isfield(settings, 'numRyRs'), fprintf(fid, 'numRyRs = %d\n', settings.numRyRs); end
		if isfield(settings, 'NUMS2S'), fprintf(fid, 'NUMS2S = %d\n', settings.NUMS2S); end
		if isfield(settings, 'BCL_S2'), fprintf(fid, 'BCL_S2 = %d\n', settings.BCL_S2); end		
		if isfield(settings, 'RyRPo_times'), fprintf(fid, 'RyRPo_times = %s\n', (sprintf("%g,",settings.RyRPo_times))(1:end-1)); end		
		if isfield(settings, 'RyRPo_levels'), fprintf(fid, 'RyRPo_levels = %s\n', (sprintf("%g,",settings.RyRPo_levels))(1:end-1)); end
		if isfield(settings, 'RyRPo_indices'), fprintf(fid, 'RyRPo_indices = %s\n', (sprintf("%g,",settings.RyRPo_indices))(1:end-1)); end
		if isfield(settings, 'NCX_clamp_times'), fprintf(fid, 'NCX_clamp_times = %s\n', (sprintf("%g,",settings.NCX_clamp_times))(1:end-1)); end		
		if isfield(settings, 'NCX_clamp_levels'), fprintf(fid, 'NCX_clamp_levels = %s\n', (sprintf("%g,",settings.NCX_clamp_levels))(1:end-1)); end	
		if isfield(settings, 'Output_Binary_Data'), fprintf(fid, 'Output_Binary_Data = %s\n', settings.Output_Binary_Data); end
		if isfield(settings, 'Load_X0'), fprintf(fid, 'Load_X0 = %s\n', settings.Load_X0); end
		if isfield(settings, 'Update_X0'), fprintf(fid, 'Update_X0 = %s\n', settings.Update_X0); end
		if isfield(settings, 'X0_File'), fprintf(fid, 'X0_File = %s\n', settings.X0_File); end
		
    if isfield(settings, 'Other')
			for ii=1:size(settings.Other,1)
				fprintf(fid, '%s\n', settings.Other(ii,:)); 
			end	
		end
    fclose(fid);
    
	% If paramname has slashes, this will be problematic, only use the last part:
	idx = strfind(paramname, '/');
	indx2 = strfind(paramname, '\\');
	if isempty(idx)
		if isempty(indx2)
			finind = 0;
		else
			finind = indx2(end);
		end
	else
		if isempty(indx2)
			finind = idx(end);
		else
			finind = max(idx(end), indx2(end));
		end
	end
	bat_fn = paramname(finind+1:end-4)
	
	fid = fopen(['run_octave_', bat_fn, '.bat'], 'w');
		fprintf(fid, 'PATH c:\\Octave\\mingw32\\bin\\\n');
			if settings.OpenMP == 0
				fprintf(fid, 'HAMM.exe %s %s CELL', paramname, outputname);
			else
				fprintf(fid, 'HAMMOpenMP.exe %s %s CELL', paramname, outputname);
			end
	fclose(fid);
	
	pause(0.3);
	
	if isfield(settings, 'hideOutput') && strcmp(settings.hideOutput, 'true')
		[status, msg] = system(['run_octave_', bat_fn]);
	else
		system(['run_octave_', bat_fn]);
	end
	
	pause(0.3);
    outputname
	
	if nargout >= 1
		if isfield(settings, 'Output_Binary_Data') && strcmp(settings.Output_Binary_Data, 'true')
			data = loadTissueData([outputname, '_', num2str(settings.bcl), '_ms.txt'], settings.NumBinaryCols);
		else
			data = dlmread([outputname, '_', num2str(settings.bcl), '_ms.txt']);
		end
		
		if exist([outputname, '_', num2str(settings.bcl), '_ms_APD.txt']) == 2
			APD = dlmread([outputname, '_', num2str(settings.bcl), '_ms_APD.txt']);
		else
			APD = NaN;
		end
	else
		data = dlmread(['', outputname, '_', num2str(settings.bcl), '_ms.txt']);
		figure;
			plot(data(:,1), data(:,2), 'LineWidth', 2);		
	end
		
end

function settings = setDefaultSettings(settings)
	if ~isfield(settings, 'bcl'), settings.bcl = 1000; end
	if ~isfield(settings, 'ft'), settings.ft = 100000; end
	if ~isfield(settings, 'startsave'), settings.startsave = 96000; end
	if ~isfield(settings, 'Output_APD'), settings.Output_APD = 'true'; end
	if ~isfield(settings, 'levelOfElectroDetail'), settings.levelOfElectroDetail = 42; end
	if ~isfield(settings, 'Adaptive_dt_Alpha'), settings.Adaptive_dt_Alpha = 0.995; end	
	if ~isfield(settings, 'OpenMP'), settings.OpenMP = 0; end	
end

