function [params, paramvals, qual] = curvefit(sfunc, xdata, ydata, x0, lb, ub, plotoptions)
		if nargin < 7, plotoptions = []; end
			if ~isfield(plotoptions, 'semilogx'), plotoptions.semilogx = 0; end
			if ~isfield(plotoptions, 'show'), plotoptions.show = 1; end
			if ~isfield(plotoptions, 'useSEMAsWeight'), plotoptions.useSEMAsWeight = 1; end
			if ~isfield(plotoptions, 'drawLinesOnTop'), plotoptions.drawLinesOnTop = 0; end
			
	sargs = symvar(sfunc);
	params = [];
	for i=1:size(sargs,1)
		if ~strcmp(sargs{i}, 'x')
			params = strvcat(params, sargs{i});
		end
	end
	
	if nargin < 6 || isempty(ub), ub = 1e10 * ones(size(params,1),1); end
	if nargin < 5 || isempty(lb), lb = -1e10 * ones(size(params,1),1); end
	if nargin < 4 || isempty(x0)
		x0supplied = false;
		while 1
			x0 = 0.5 * (ub + lb) + sqrt((1 + abs(0.5 * (ub + lb)))) .* randn(size(params,1),1); 
			if all(x0 >= lb) && all(x0 <= ub)
				break;
			end
		end
	else
		x0supplied = true;
	end
	
	global add_curvefit_data;
	if plotoptions.useSEMAsWeight == 1
		add_curvefit_data = {sfunc, xdata, ydata, params, lb, ub};
	else
		add_curvefit_data = {sfunc, xdata, ydata(:,1), params, lb, ub};
	end
		
	params;
	paramvals = x0;
	
	opts(1) = plotoptions.show;
	opts(2) = 1E-3;
	opts(6) = 0;
	opts(5) = 0;
	opts(10) = 500;
	
	%feval('deterr', paramvals, sfunc, xdata, ydata, params)
	
	opts = optimset('MaxFunEvals',100);
	paramvals = fminsearch('curvefit_deterr', x0, opts);
	
	[errfnl, yinterp] = curvefit_deterr(paramvals);
		qual.err = errfnl;
		qual.SSE = sum((ydata(:,1)  - yinterp).^2);
		%qual.rsquare = corrcoef(ydata(:,1), yinterp)^2;
		
	if plotoptions.show == 1
		if plotoptions.semilogx == 1
			xinterp = logspace(log(min(xdata)),log(max(xdata)), max(100, length(xdata)))';
		else
			xinterp = linspace(min(xdata),max(xdata), max(100, length(xdata)))';
		end
			
		if plotoptions.useSEMAsWeight == 1
			add_curvefit_data = {sfunc, xinterp, ydata, params, lb, ub};
		else
			add_curvefit_data = {sfunc, xinterp, ydata(:,1), params, lb, ub};
		end
		[temp_x0, yinterp_x0] = curvefit_deterr(x0);
		[temp, yinterp] = curvefit_deterr(paramvals);
	
		figure;
			if plotoptions.semilogx == 1
				if x0supplied
					semilogx(xdata, ydata(:,1), 'ok', 'LineWidth', 2, xinterp, yinterp, '-', xinterp, yinterp_x0, 'LineWidth', 2);
				else
					semilogx(xdata, ydata(:,1), 'ok', 'LineWidth', 2, xinterp, yinterp, '-', 'LineWidth', 2);
				end
			else
				if plotoptions.drawLinesOnTop == 1
					hold on;
					plot(xdata, ydata(:,1), 'ok', 'LineWidth', 1);
					if x0supplied
						plot(xinterp, yinterp, '-', 'LineWidth', 3, xinterp, yinterp_x0, '-', 'LineWidth', 3);
					else
						plot(xinterp, yinterp, '-', 'LineWidth', 3);
					end
				else
					if x0supplied
						plot(xdata, ydata(:,1), 'ok', 'LineWidth', 2, xinterp, yinterp, '-', 'LineWidth', 2, xinterp, yinterp_x0, '-', 'LineWidth', 2);
					else
						plot(xdata, ydata(:,1), 'ok', 'LineWidth', 2, xinterp, yinterp, '-', 'LineWidth', 2);
					end
				end
			end
			
			if size(ydata,2) == 2
				hold on;
				h = errorbar(xdata, ydata(:,1), ydata(:,2), '~');
				set(h, 'linestyle', 'none', 'Color', 'black');
			end
	end

end