function [xfinal, errfinal] = runLargeCtlOpt(show_it, loadDataFromFileOnly, orderOfSims, output_basename, pAFFactors, simRyR2Gaps, RyR2Heterogeneity, simAxialTubule, simAxialTubuleRyR2Hyperphos)
	if nargin < 1, show_it = 0; end
	if nargin < 2, loadDataFromFileOnly = 0; end
	if nargin < 3, orderOfSims = [1;2;3]; end
	if nargin < 4, output_basename = []; end
	if nargin < 5, pAFFactors = [1; 1; 1]; end
	if nargin < 6, simRyR2Gaps = 0; end %Default = 50 segment sims
	if nargin < 7, RyR2Heterogeneity = 0; end % Default: no heterogeneity
	if nargin < 8, simAxialTubule = 0; end % Default: no axial tubules
	if nargin < 9, simAxialTubuleRyR2Hyperphos = 0; end
	
	X0 = 0;
	maxfunevals = 0;
	indices = 1;
	ident = '';
	
	global add_opt_data;
		add_opt_data = {X0, indices, show_it, loadDataFromFileOnly, orderOfSims, output_basename, pAFFactors, ident, simRyR2Gaps, RyR2Heterogeneity, simAxialTubule, simAxialTubuleRyR2Hyperphos};
	
	opts(1) = 1;
	opts(2) = 1E-3;
	opts(6) = 0;
	opts(5) = 0;
	opts(10) = maxfunevals;
	
	if maxfunevals > 0
		xopt = fminsearch(@determineError, zeros(1,length(indices)), opts);
		
		add_opt_data = {X0, indices, 1, loadDataFromFileOnly, orderOfSims, output_basename, pAFFactors, ident};
		errfinal = determineError(xopt);
	else
		xopt = zeros(1, length(indices));
		errfinal = determineError(xopt);
	end	
	
	xfinal = zeros(1,length(X0));	
	xfinal(indices) = xopt;
	xfinal = (1 + xfinal) .* X0;
end

function errtot = determineError(params)
	global add_opt_data;
		X0_l = add_opt_data{1};
		indices_l = add_opt_data{2};
		show_it = add_opt_data{3};
		loadDataFromFileOnly = add_opt_data{4};
		orderOfSims = add_opt_data{5};
		output_basename = add_opt_data{6};
		pAFFactors = add_opt_data{7};
		ident = add_opt_data{8};
		simRyR2Gaps = add_opt_data{9};
		RyR2Heterogeneity = add_opt_data{10};
		simAxialTubule = add_opt_data{11};
		simAxialTubuleRyR2Hyperphos = add_opt_data{12};		
		
	cparams = zeros(1,length(X0_l));
	cparams(indices_l) = params;
	cparams = (1 + cparams) .* X0_l
	
	err_leak = 0; err_cat = 0; err_ical = 0; err_sparks = 0; err_currentclamp = 0;	
	for aa=1:length(orderOfSims)
		switch(orderOfSims(aa))
			case 1
				[err_cat, err_ical] = runCaTICaLSims([], show_it, cparams, loadDataFromFileOnly, output_basename, pAFFactors, ident, simRyR2Gaps, RyR2Heterogeneity, simAxialTubule, simAxialTubuleRyR2Hyperphos);
			case 2 
				err_jphysiol = runCurrentClampJPhysiol([], show_it, cparams, loadDataFromFileOnly, output_basename, pAFFactors, ident, simRyR2Gaps, RyR2Heterogeneity, simAxialTubule, simAxialTubuleRyR2Hyperphos);
		endswitch
	end
		
	err = [err_leak; err_cat; err_ical; err_sparks; err_currentclamp];
	errtot = sum(err .^ 2)
	
	if size(params,1) > size(params,2), params = params'; end	
	if abs(pAFFactors(1) - 1) < 1E-10 && abs(pAFFactors(2) - 1) < 1E-10
		dlmwrite('runLargeCtlOpt_120913.score', [errtot, cparams, err'], '-append');	
	else
		if size(pAFFactors,1) > size(pAFFactors,2), pAFFactors = pAFFactors'; end
		dlmwrite('runLargeCtlOpt_120913_pAF.score', [errtot, cparams, pAFFactors, err'], '-append');	
	end
end

function [err_cat, err_ical] = runCaTICaLSims(settings, show_it, cparams, loadDataFromFileOnly, output_basename, pAFFactors, ident, simRyR2Gaps, RyR2Heterogeneity, simAxialTubule, simAxialTubuleRyR2Hyperphos)
	% Settings
	% =========================================================================    
  settings.levelOfElectroDetail = 53;
	settings.run2D = 0;
	settings.DT_MAX = 0.2;
	settings.Output_APD = 'true';		
	settings.Adaptive_dt_Alpha = 0.995;
	settings.skip = 7;
	settings.update = 100000;
	settings.applyVoltageClamp = 'true';
	
	% Based on Data Supplement of Voigt, Heijman et al. Circulation 2014:
	settings.RyR_NP_Params = [pAFFactors(7)*0.2,pAFFactors(8),12,0.63,9.0E-4,0.8E-4,pAFFactors(4)*0.0035,6.3,0.0,1.0,1.0,pAFFactors(1) * 0.007,20,-1,-1,-1,-1,-1,-1,-1,1.75,1.0,50,1.5,0.5];		% cparams(16) = 5.75E-4
		
    %%%% These are used for local hyperphosphorylation of RyR2
    RyR_NP_Params = [pAFFactors(7)*0.2,pAFFactors(8),12,0.63,9.0E-4,0.8E-4,pAFFactors(4)*0.0035,6.3,0.0,1.0,1.0,pAFFactors(1) * 0.007,20,-1,-1,-1,-1,-1,-1,-1,1.75,1.0,50,1.5,0.5];
    RyR_P_Params = [pAFFactors(7)*0.2*2,pAFFactors(8)*1.09,12,0.63,9.0E-4,0.8E-4,pAFFactors(4)*0.0035*3,6.3,0.0,1.0,1.0,pAFFactors(1) * 0.007,20,-1,-1,-1,-1,-1,-1,-1,1.75,1.0,50,1.5,0.5];

	if simRyR2Gaps >=1
		settings.numRyRs = pAFFactors(1) * 198000 * 14; %settings.numRyRs = pAFFactors(1) * 198000 * 3;
		settings.IupB = 1 - pAFFactors(2) + pAFFactors(2) * 0.4;
		settings.tau_diff_Cai_Domain = pAFFactors(5)*0.6; 	
		settings.tau_diff_Casr_Domain = 5;
		settings.tau_diff_Cai_Segment = pAFFactors(5)*0.6;
		settings.tau_diff_Casr_Segment = 5;		
		settings.tau_diff_Casrs_Segment = 0.22; %0.15;
		settings.tau_diff_Casrs_Domain = 0.16; 
    if pAFFactors(4) <= 2
		  settings.tau_diff_SRS_i = 10; %prev.16
    else 
      settings.tau_diff_SRS_i = 10; %prev.16
    end 
		settings.factDiffJuncSl = 0.01;		settings.factDiffSlMyo = 1.75;
		
		settings.N_Segments = 100; settings.N_CaDomains = 18; 
  else
		settings.numRyRs = pAFFactors(1) * 198000 * 1.9;
		settings.IupB = 1 - pAFFactors(2) + pAFFactors(2) * 0.4;
		settings.tau_diff_Cai_Domain = pAFFactors(5)*0.6; 	
		settings.tau_diff_Casr_Domain = 15;
		settings.tau_diff_Cai_Segment = pAFFactors(5)*0.6;
		settings.tau_diff_Casr_Segment = 15;		
		settings.tau_diff_Casrs_Segment = 0.3; %0.15;
		settings.tau_diff_Casrs_Domain = 0.125; 
    if pAFFactors(4) <= 2
		  settings.tau_diff_SRS_i = 12; %prev.16
    else 
      settings.tau_diff_SRS_i = 12; %prev.16
    end 
		settings.factDiffJuncSl = 0.01;		settings.factDiffSlMyo = 1.75;
		
		settings.N_Segments = 50; settings.N_CaDomains = 18; 
  end 
    
	settings.INaCaB = -0.35;
	settings.ICabB = -0.85;
	settings.allowLongitudinalSRSDiff = 'true';
		
	%Based on Voigt, Heijman et al. Circulation 2014 data supplement
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,9.45,70,49.10,10.349,26.553,0.213,10.807,17.5,3,0.2474,13.825,6.3836,0.001,14.9186,1E-6,1.7E-4,1.1E-3,0.1,-1]; % Based on Voigt et al. Circ 2014
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,8.50,70,49.10,10.349,26.553,0.213,10.807,17.5,3,0.2474,13.825,6.3836,0.001,14.9186,1E-6,1.7E-4,1.1E-3,0.1,-1]; % Based on Voigt et al. Circ 2014 with modification in ICaL act (06072018)
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,9.45,70,65,6,12,0.213,10.807,27.5,5,0.2474,43.825,45,0.001,5.0,1E-6,1.7E-4,5.0E-3,0.1,-1];
	settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,9.45,70,65,6,12,0.213,10.807,27.5,5,0.2474,43.825,45,0.001,5.0,1E-6,1.7E-4,5.0E-3,0.1,-1]; %from Heijman et al Jphysiol 2016	
	
	settings.IK1B = 1.0; settings.IKrB = 1.0; settings.ITo1B = 1.0; settings.IKsB = 1.0; settings.IKpB = 1.0; settings.IKurB = 1.0;
	settings.IClbB = 1.0; settings.INaKB = 1.0;	settings.INaB = 1.0;
	settings.Ca_o = 2.0;
	settings.Fluo3 = 0.0; % 0.025;
	settings.cellType = 2;
	settings.Clamp_Na_i = 8.5;
	
	settings.Output_Binary_Data = 'true';
	settings.numVarsPerCaDomain = 4;
	if settings.levelOfElectroDetail == 52, settings.numVarsPerCaDomain = 8; end
	if settings.levelOfElectroDetail == 53, settings.numVarsPerCaDomain = 5; end
	settings.startofCaData = 19;
	
	settings.segOutputRes = 1; settings.domOutputRes = 1;
	settings.NumBinaryCols = (settings.startofCaData - 1) + settings.numVarsPerCaDomain * (settings.N_Segments / settings.segOutputRes) * (settings.N_CaDomains / settings.domOutputRes); 
	settings.RyRModelType = 3;
	settings.hideOutput = 'false';
    
    numBeats = 20;
	settings.startsave = max(0,(numBeats-3)*2064);
	settings.NUMS2S = 1;
	settings.BCL_S2 = 15000;
    
    %ryr_vals= dlmread('intensity3.txt');
    ryr_vals= Diff_distributions(settings.N_Segments, settings.N_CaDomains, RyR2Heterogeneity, simRyR2Gaps);
    ryr_text = sprintf('%f,', ryr_vals'(:));
    dlmwrite([output_basename, '_RyRStructure_Det.txt'], ryr_vals);
    
	if simAxialTubule >= 1
		% ICaL_Distribution  
		ical_structure = ones(settings.N_Segments,settings.N_CaDomains);
		ical_structure(:,2:17)=0;
		%ical_structure(5:24,13)=1;
		%random axial tubule --------------------------------------------------
		ical_structure(2,11:12)=1;
		ical_structure(3,9:11)=1;
		ical_structure(2:8,17)=1;
		ical_structure(2,18)=1;
		ical_structure(6:13,10)=1;
		ical_structure(10,12)=1;
		ical_structure(13:28,9)=1;
		ical_structure(14,8)=1;
		ical_structure(13:19,5)=1;
		ical_structure(19:35,4)=1;
		ical_structure(29,8)=1;
		ical_structure(28:29,10)=1;
		ical_structure(29,12)=1;
		ical_structure(24:28,13)=1;
		ical_structure(30:46,11)=1;
		ical_structure(46,9)=1;
		ical_structure(36,5:10)=1;
		ical_structure(34,14)=1;
		ical_structure(46:64,10)=1;
		ical_structure(51,11:13)=1;
		ical_structure(46:47,17)=1;
		ical_structure(56,11)=1;
		ical_structure(57:60,12)=1;
		ical_structure(64:78,9)=1;
		ical_structure(65,11)=1;
		ical_structure(66,12)=1;
		ical_structure(67:69,13)=1;
		ical_structure(69,14)=1;
		ical_structure(79,8)=1;
		ical_structure(79:89,10)=1;
		ical_structure(52,1)=1;
		ical_structure(53:54,2)=1;
		ical_structure(55,3)=1;
		ical_structure(82,18)=1;
		ical_structure(89:91,7)=1;
		ical_structure(91,8)=1;
		ical_structure(55:59,4)=1;
		%ical_structure(5:24,5)=1;
		%ical_structure(settings.N_Segments-25:settings.N_Segments-6,5)=1;
		ical_vals = ical_structure;
		%ical_vals = '_icalStructure.txt';
		
	else
		 ical_vals = [1, zeros(1,settings.N_CaDomains-2), 1];
	end 
	ical_text = sprintf('%f,', ical_vals'(:));
    dlmwrite([output_basename, '_icalStructure_Det.txt'], ical_vals);

	if simAxialTubuleRyR2Hyperphos >= 1
		% Local Hyperphosphorylation of RyR2:
		total_RyR_params = [];
		for ii=1:settings.N_Segments
			for jj=1:settings.N_CaDomains
			  %if no axial tubule here:
			  %if (jj >= 11 && jj <=12 && ii==2) || (jj >=9 && jj <=11 && ii==3) || (jj==17 && ii>=2 && ii<=8) || (jj==18 && ii==2) || (jj == 10 && ii>=6 && ii<=13) || (jj == 12 && ii==10) || (jj == 9 && ii>=13 && ii<=28) || (jj == 8 && ii==14) || (jj == 5 && ii>=13 && ii<=19) || (jj == 4 && ii>=19 && ii<=35) || (jj == 8 && ii==29) || (jj == 10 && ii>=28 && ii<=29) || (jj == 12 && ii==29) || (jj == 13 && ii>=24 && ii<=28) || (jj == 11 && ii>=30 && ii<=46) || (jj == 9 && ii==46) || (jj >=5 && jj<=10 && ii==36) || (jj == 14 && ii==34) || (jj == 10 && ii>=46 && ii<=64) || (jj >= 11 && jj <=13 && ii==51) || (jj == 17 && ii>=46 && ii<=47) || (jj == 11 && ii==56) || (jj == 12 && ii>=57 && ii<=60) || (jj == 9 && ii>=64 && ii<=78) || (jj == 11 && ii==65) || (jj == 12 && ii==66) || (jj == 13 && ii>=67 && ii<=69) || (jj == 14 && ii==69) || (jj == 8 && ii==79) || (jj == 10 && ii>=79 && ii<=89) || (jj == 1 && ii==52) || (jj == 2 && ii>=53 && ii<=54) || (jj == 3 && ii==55) || (jj == 18 && ii==82) || (jj == 7 && ii>=89 && ii<=91) || (jj == 8 && ii==91) || (jj == 4 && ii>=55 && ii<=59)    
			  %if (jj == 13 && ii>=5 && ii<=24) || (jj == 5 && ii>=settings.N_Segments-25 && ii<=settings.N_Segments-6)
			  if jj > 1 && jj < 18 && ical_structure(ii,jj) == 1
				total_RyR_params = [total_RyR_params, RyR_P_Params];
			  else
				total_RyR_params = [total_RyR_params, RyR_NP_Params];
			  end
			end
		end
		settings.RyR_NP_Params = total_RyR_params;
	end
	
	settings.Other = strvcat(['SERCA_Kmr = ', num2str(pAFFactors(6)*1.0)], ...
		['SERCA_Kmf_NP = ', num2str(pAFFactors(3)*6.25E-4)], ...
		'Update_X0 = false', ...
		'Load_X0 = false', ...
		['X0_File = X0_runLargeCtlOpt_LongPaced', ident, '.txt'], ...			
		'VjuncFact = 0.001', ...
		['Ca_Buff_Factor = 1.0'], ...
		['ICaLDistribution = ', ical_text(1:end-1)], ...
		%['ICaLDistribution = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1'], ...
		['RyRDistribution =  ', ryr_text(1:end-1)], ...
		['NCXDistribution =  1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1'], ...
		['SERCADistribution =1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1'] ...
	);
					
		settings.RyRPo_times = [numBeats*2064+3000, numBeats*2064+3004, numBeats*2064+3006, numBeats*2064+3008, numBeats*2064+3010, numBeats*2064+13000, 1E6];
		settings.RyRPo_levels = [0.1, 0.3, 0.6, 0.8, 0.99, -1, -1];
		
		if loadDataFromFileOnly ~= 1
			data_SR = runICaLVoltageClamp(numBeats, settings, ['Parameters_runLargeCtlOpt_Stoch', ident, '.txt'], ['Output_runLargeCtlOpt_Stoch', ident]);
		else	
			data_SR = loadTissueData(['Output_runLargeCtlOpt_Stoch', ident, '_2064_ms.txt'], settings.NumBinaryCols);
		end
		settings.N_Segments = max(settings.N_Segments / settings.segOutputRes,1); 
		settings.N_CaDomains = max(settings.N_CaDomains / settings.domOutputRes,1); 
		
		% data analysis
		% ------------------------------------
			[val, ind_start_normal] = min(abs(data_SR(:,1) - 3*2064));
			[val, ind_start_tetracaine] = min(abs(data_SR(:,1) - (3*2064+2064)));
			[val, ind_end_tetracaine] = min(abs(data_SR(:,1) - (3*2064+2064+20000)));
			
			diast_level_normal_SR = min(data_SR(ind_start_normal:ind_start_tetracaine,3))
			syst_level_normal_SR = max(data_SR(ind_start_normal:ind_start_tetracaine,3))
			diast_level_tetracaine_SR = min(data_SR(ind_start_tetracaine:ind_end_tetracaine,3))
			[syst_level_caffeine_SR, ind_peak_caffeine] = max(data_SR(ind_end_tetracaine:end,3))
			ca_leak_SR = diast_level_normal_SR - diast_level_tetracaine_SR
			ca_load_SR = syst_level_caffeine_SR - diast_level_normal_SR
			
			Exp_leak_Ctl = [59.61945032, 7.293868922];
			Exp_leakload_Ctl = [4.451476793, 0.548523207];

	% data analysis - VCP
		% ------------------------------------
		plotoptions.show = 0;
				
		[val, endindex_SR] = min(abs(data_SR(:,1) - 7000));
		%[val, endindex_new_SR] = min(abs(data_new_SR(:,1) - 7000));
		
		Cai_diast_SR = 1000 * min(data_SR(1:endindex_SR,3))
		Cai_syst_SR = 1000 * max(data_SR(1:endindex_SR,3))
		Casr_diast_SR = 1000 * max(data_SR(1:endindex_SR,4))
		Casr_syst_SR = 1000 * min(data_SR(1:endindex_SR,4))
    
		[val, ind] = min(abs(data_SR(1:endindex_SR,1) - 4200));
		[maxval, maxind] = max(data_SR(1:ind,3));
		Cai_amp_SR = 1000 * (maxval - min(data_SR(1:ind,3)))
    
   
    %min_cai= min(data_SR(ind_s:ind_e,3))
    %[max_cai, maxind]= max(data_SR(ind_s:ind_e,3))
    %ca_i_amp_new= max_cai-min_cai
    
		if data_SR(maxind,1) > 2000
			Cai_timetopeak_SR = data_SR(maxind,1) - (200 + 2064)
		else
			Cai_timetopeak_SR = data_SR(maxind,1) - (200 + 0)
		end		
		[val, ind] = min(abs(data_SR(1:endindex_SR,1) - (data_SR(maxind,1)+1500)));
		dCadt = (data_SR(maxind+1:ind,3) - data_SR(maxind:ind-1,3)) ./ (data_SR(maxind+1:ind,1) - data_SR(maxind:ind-1,1));
		posind = find(dCadt > 0.01);
		
		if ~isempty(posind)
			ind = posind(1) + maxind - 10;
		end
		
		maxind = maxind + 30;
		[params, paramvals, qual] = curvefit('a * exp(-x / b) + c', data_SR(maxind:ind,1)-data_SR(maxind), 1000*data_SR(maxind:ind,3), [Cai_amp_SR; 300; Cai_diast_SR], [0; 0; 0], [], plotoptions);
		Cai_tau_SR = paramvals(2)
		maxind = maxind - 30;
		
		Exp_Cai_diast_SR = [256.0, 21.94]; %Voigt et al. Circulation 2012: [222.44, 15.63];
		Exp_Cai_diast_pAF = [253.1, 40.90]; Exp_Cai_diast_cAF = [277.76, 27.78];

		Exp_Cai_syst_SR = [448.8, 30.50]; %Voigt et al. Circulation 2012: [504.86, 35.66];
		Exp_Cai_syst_pAF = [552.2, 75.16]; Exp_Cai_syst_cAF = [417.03, 36.21];
		
		Exp_Cai_amp_SR = [192.8, 15.61]; %Voigt et al. Circulation 2012: [263.21, 21.37];
		Exp_Cai_amp_pAF = [299.1, 57.3]; Exp_Cai_amp_cAF = [139.27, 13.51];
		
		Exp_Cai_timetopeak_SR = [49.84, 3.16];
		Exp_Cai_timetopeak_cAF = [58.68, 3.64];
		
		Exp_Cai_tau_SR = [298.4, 23.07]; %Voigt et al. Circulation 2012: [228.03, 15.31];
		Exp_Cai_tau_pAF = [217.7, 19.49]; Exp_Cai_tau_cAF = [290.91, 21.95];
		
		% data analysis - ICaL
		% ------------------------------------
		[val, ind1] = min(abs(data_SR(:,1) - 200))
		[val, ind] = min(abs(data_SR(:,1) - 299))
		[ICaL_peak_SR, ind_peak] = max(abs(data_SR(ind1:ind,9)));
		ICaL_int_SR = trapz(data_SR(ind1:ind,1),-(data_SR(ind1:ind,9)-data_SR(ind,9)));
			
		[params, fit_vals, qual] = curvefit('-a*exp(-x/b)-c*exp(-x/d)+f', data_SR(ind1+5:ind-29,1)-data_SR(ind1+5,1), data_SR(ind1+5:ind-29,9), [5.6;3;1.4;40;0], [0;0;0;0;-10], [], plotoptions);
		ICaL_fitparams_SR = [fit_vals(2), fit_vals(4), fit_vals(1) / (fit_vals(1) + fit_vals(3))];
		
		Exp_ICaL_peak_SR = [6.283, 0.6095]; %Voigt et al. Circulation 2012: [6.20, 0.44];
		Exp_ICaL_peak_pAF = [6.288, 1.085]; Exp_ICaL_peak_cAF = [3.52, 0.34];
		
		Exp_ICaL_int_SR = [97.05, 7.393]; %Voigt et al. Circulation 2012: [82.53, 5.81];
		Exp_ICaL_int_pAF = [84.38, 12.41]; Exp_ICaL_int_cAF = [62.02, 4.27];
			
		Exp_ICaL_Tau_Inact_slow_SR = [43.42, 2.8];
		Exp_ICaL_Tau_Inact_fast_SR = [3.87, 0.31];
		Exp_ICaL_Inact_frac_fast_SR = [0.69, 0.02];
		
		% data analysis - caffeine
		% ------------------------------------
		[val, ind_start_SR] = min(abs(data_SR(:,1) - 3*2064 - 3000));
		[val, ind_stop_SR] = min(abs(data_SR(:,1) - 3*2064 - 9000));
		[maxval_CaT_SR, maxind_CaT_SR] = max(data_SR(ind_start_SR:ind_stop_SR,3));
		[minval_CaT_SR, minind_CaT_SR] = min(data_SR(ind_start_SR-200:ind_start_SR,3));
		[maxval_CaTsr_SR, maxind_CaT_SR] = max(data_SR(ind_start_SR:ind_stop_SR,4));
		[minval_CaTsr_SR, minind_CaT_SR] = min(data_SR(ind_start_SR-200:ind_start_SR,4));
		[maxval_NCX_SR, maxind_NCX_SR] = min(data_SR(ind_start_SR:ind_stop_SR,9));
		NCX_int_SR = trapz(data_SR(ind_start_SR:ind_stop_SR,1),(data_SR(ind_start_SR:ind_stop_SR,9) - data_SR(ind_stop_SR,9))) / 1000
		
		[params, paramvals, qual] = curvefit('a * exp(-x / b) + c', data_SR((maxind_CaT_SR+ind_start_SR):(maxind_CaT_SR+ind_stop_SR),1)-data_SR(maxind_CaT_SR+ind_start_SR,1), 1000*data_SR((maxind_CaT_SR+ind_start_SR):(maxind_CaT_SR+ind_stop_SR),3), [maxval_CaT_SR; 1000; minval_CaT_SR], [0; 0; 0], [], plotoptions);
		cCaT_tau_SR = paramvals(2)
		
		[val, thresh_index_SR] = min(abs(data_SR((maxind_CaT_SR+ind_start_SR):end,3) - 2.0)); thresh_index_SR = maxind_CaT_SR + ind_start_SR + thresh_index_SR;
		[params, NCX_Ca_dep_SR, qual] = curvefit('-a * x + b', data_SR(thresh_index_SR:ind_stop_SR,3), data_SR(thresh_index_SR:ind_stop_SR,9), [1.0; -0.3], [], [], plotoptions);
		
		Exp_cCaT_tau_SR = [821.0, 103.6] %Voigt et al. Circulation 2012: [1287.05, 361.83];
		Exp_cCaT_tau_pAF = [1268.0, 338.9]; Exp_cCaT_tau_AF = [473.10, 52.25]; 
		Exp_cCaT_amp_SR = [1363, 204.9] ./ 1000; %Voigt et al. Circulation 2012: [1359.68, 228.07] / 1000;
		Exp_cCaT_amp_pAF = [2227, 183.0] ./ 1000; Exp_cCaT_amp_cAF = [1200.28, 83.67] / 1000;
		Exp_INCX_peak_SR = [-1.764, 0.2304]; %Voigt et al. Circulation 2012: [-1.26, 0.21];
		Exp_INCX_peak_pAF = [-1.952, 0.3766]; Exp_INCX_peak_cAF = [-2.09, 0.27];
		Exp_INCX_int_SR = [-1.109, 0.1143]; %Voigt et al. Circulation 2012: [-0.81, 0.08]; % in pC / pF = Int(INCX) * 1E-3
		Exp_INCX_int_pAF = [-2.716, 0.8173]; Exp_INCX_int_cAF = [-0.78, 0.10]; % in pC / pF = Int(INCX) * 1E-3
		Exp_INCX_Cai_dep_SR = [1.041, 0.1623]; %Voigt et al. Circulation 2012: [0.80, 0.11, -0.12, 0.03];
		Exp_INCX_Cai_dep_pAF = [0.8850, 0.1415]; Exp_INCX_Cai_dep_cAF = [1.38, 0.17, -0.24, 0.09];
		
		[val, endindex_SR_end] = min(abs(data_SR(:,1) - 20000));
	
		% Visualization
		% ------------------------------------
		if show_it >= 1
		xlimplot = [0, 400];
		figure;
			set(gcf, 'Position', [200, 100, 1000, 900]);
			%Exp_Traces_Ctl = dlmread('Ms_Human_Atrial_Model/Suppl_Fig_2_ICaL_Validation/09o23010_SR_Baseline.txt'); % Capacitance: 101.56 pF
			
			subplot(9,3,[1,2,4,5]);
				plot(data_SR(:,1), data_SR(:,2), '-k', 'LineWidth', 2); xlim(xlimplot);			
			
			%subplot(9,3,[7,8,10,11]);
				%plot(Exp_Traces_Ctl(:,1).*1000, Exp_Traces_Ctl(:,2) ./ 101.56, '-k', 'LineWidth', 2); xlim(xlimplot); ylim([-10, 1.0]);
			
			subplot(9,3,[13,14,16,17]);
				plot(data_SR(:,1), data_SR(:,9), '-k', 'LineWidth', 2); xlim(xlimplot); ylim([-10, 1.0]);
			if abs(pAFFactors(1) - 1) < 1E-10 && abs(pAFFactors(2) - 1) < 1E-10
				subplot(9,3,[3,6,9]); plotSRAFBars(Exp_ICaL_peak_SR, NaN, ICaL_peak_SR, NaN, 'Peak ICa,L');
				subplot(9,3,[12,15,18]); plotSRAFBars(Exp_ICaL_int_SR, NaN, ICaL_int_SR, NaN, 'Int ICa,L');			
			else
				subplot(9,3,[3,6,9]); plotSRAFBars(Exp_ICaL_peak_pAF, NaN, ICaL_peak_SR, NaN, 'Peak ICa,L');
				subplot(9,3,[12,15,18]); plotSRAFBars(Exp_ICaL_int_pAF, NaN, ICaL_int_SR, NaN, 'Int ICa,L');			
			end
			subplot(9,3,[19,20,22,23,25,26]); h = bar([Exp_ICaL_Tau_Inact_slow_SR(1), ICaL_fitparams_SR(2); Exp_ICaL_Tau_Inact_fast_SR(1), ICaL_fitparams_SR(1)]);
				set (h(1), 'facecolor', 'w', 'LineWidth', 2);
				set (h(2), 'facecolor', 'k', 'LineWidth', 2);
				title('Tau ICa,L Inact');
			subplot(9,3,[21,24,27]); plotSRAFBars(Exp_ICaL_Inact_frac_fast_SR, NaN, ICaL_fitparams_SR(3), NaN, 'Frac. ICaL Fast');
			
		xlimplot = [2500, 15000];
		figure;
			set(gcf, 'Position', [200, 100, 1200, 900]);
			
			subplot(7,5,[1,2,3,4,5]);
				plot(data_SR(:,1), data_SR(:,2), '-k', 'LineWidth', 2); xlim(xlimplot);	grid off;	ylim= [-100 20]; %set(gca, 'YTick', []); set(gca, 'XTick', []);
				csvwrite('Vm_validation_protocol_henry.txt', [data_SR(:,1), data_SR(:,2)])
        
			subplot(7,5,[6:10]);
				plot(data_SR(:,1), data_SR(:,3), '-k', 'LineWidth', 2); xlim(xlimplot); grid off; ylim= [0 2]; %set(gca, 'YTick', []); set(gca, 'XTick', []);	
        csvwrite('Cai_validation_protocol_henry.txt', [data_SR(:,1), data_SR(:,3)])
        
			subplot(7,5,[11,12,13,14,15]);
				plot(data_SR(:,1), data_SR(:,9), '-k', 'LineWidth', 2); xlim(xlimplot);
			  csvwrite('ina_maybe_validation_protocol.txt', [data_SR(:,1), data_SR(:,9)])
        
			if abs(pAFFactors(1) - 1) < 1E-10 && abs(pAFFactors(2) - 1) < 1E-10
				subplot(7,5,[16,17,18,21,22,23]);
					h = bar([Exp_Cai_diast_SR(1), Cai_diast_SR; Exp_Cai_syst_SR(1), Cai_syst_SR; Exp_Cai_amp_SR(1), Cai_amp_SR]);
					set (h(1), 'facecolor', 'w', 'LineWidth', 2);
					set (h(2), 'facecolor', 'k', 'LineWidth', 2);
					title('Intracellular Ca2+');
				
				subplot(7,5,[19,24]); plotSRAFBars(Exp_Cai_tau_SR, NaN, Cai_tau_SR, NaN, 'CaT Decay');
        Cai_tau_SR =Cai_tau_SR
				subplot(7,5,[20,25]); plotSRAFBars(Exp_Cai_timetopeak_SR, NaN, Cai_timetopeak_SR, NaN, 'CaT Time to Peak');
				subplot(7,5,[26,31]); plotSRAFBars(Exp_cCaT_amp_SR, NaN, (maxval_CaT_SR-minval_CaT_SR), NaN, 'cCaT Ampl.');
        cCaT_amp= maxval_CaT_SR-minval_CaT_SR
        cCaTsr_sys= minval_CaTsr_SR
        cCaTsr_dias= maxval_CaTsr_SR
				subplot(7,5,[27,32]); plotSRAFBars(Exp_cCaT_tau_SR, NaN, cCaT_tau_SR, NaN, 'cCaT Decay');
        cCaT_tau_SR = cCaT_tau_SR
				subplot(7,5,[28,33]); plotSRAFBars(-Exp_INCX_peak_SR, NaN, -maxval_NCX_SR, NaN, 'Max NCX');
				subplot(7,5,[29,34]); plotSRAFBars(-Exp_INCX_int_SR, NaN, -NCX_int_SR, NaN, 'Int NCX');
			else
				subplot(7,5,[16,17,18,21,22,23]);
					h = bar([Exp_Cai_diast_pAF(1), Cai_diast_SR; Exp_Cai_syst_pAF(1), Cai_syst_SR; Exp_Cai_amp_pAF(1), Cai_amp_SR]);
					set (h(1), 'facecolor', 'w', 'LineWidth', 2);
					set (h(2), 'facecolor', 'k', 'LineWidth', 2);
					title('pAF Intracellular Ca2+');
				
				subplot(7,5,[19,24]); plotSRAFBars(Exp_Cai_tau_pAF, NaN, Cai_tau_SR, NaN, 'CaT Decay');
				subplot(7,5,[20,25]); plotSRAFBars(Exp_Cai_timetopeak_SR, NaN, Cai_timetopeak_SR, NaN, 'CaT Time to Peak');
				subplot(7,5,[26,31]); plotSRAFBars(Exp_cCaT_amp_pAF, NaN, (maxval_CaT_SR-minval_CaT_SR), NaN, 'cCaT Ampl.');
				subplot(7,5,[27,32]); plotSRAFBars(Exp_cCaT_tau_pAF, NaN, cCaT_tau_SR, NaN, 'cCaT Decay');
				subplot(7,5,[28,33]); plotSRAFBars(-Exp_INCX_peak_pAF, NaN, -maxval_NCX_SR, NaN, 'Max NCX');
				subplot(7,5,[29,34]); plotSRAFBars(-Exp_INCX_int_pAF, NaN, -NCX_int_SR, NaN, 'Int NCX');
			end
		end			

		% Output
		% ===================================================	
		if ~isempty(output_basename)
			dlmwrite([output_basename, '_Traces_Ctl.txt'], data_SR(:,1:9), '\t');
			
			rowindex = 2;
			[val, ind_s] = min(abs(data_SR(:,1) - xlimplot(1)));
			[val, ind_e] = min(abs(data_SR(:,1) - xlimplot(2)));
			Ca_i_data_SR = getDataMatrix(data_SR, startofCaData, settings.N_Segments, settings.N_CaDomains, numVarsPerCaDomain);
			Ca_i_data_SR = interp1(data_SR(:,1), Ca_i_data_SR(:,((rowindex-1)*settings.N_CaDomains+1):(rowindex*settings.N_CaDomains)), (data_SR(ind_s,1):1:data_SR(ind_e,1))');
			createMeshDataFile([(data_SR(ind_s,1):1:data_SR(ind_e,1))', 1E6 .* Ca_i_data_SR], [output_basename, '_Traces_LineScan_CaT_Ctl.txt']);
			
			dlmwrite([output_basename, '_Bar_SR_Ca_Leak_Tecplot.txt'], createValidationBarsTecplot(Exp_leak_Ctl, NaN, 1000*ca_leak_SR, NaN), '\t');
			dlmwrite([output_basename, '_Bar_SR_Ca_Leak_Prism.txt'], createValidationBarsPrism(Exp_leak_Ctl, NaN, 1000*ca_leak_SR, NaN), '\t');
			
			dlmwrite([output_basename, '_Bar_SR_Ca_LeakLoad_Tecplot.txt'], createValidationBarsTecplot(Exp_leakload_Ctl, NaN, 100*(ca_leak_SR/ca_load_SR), NaN), '\t');
			dlmwrite([output_basename, '_Bar_SR_Ca_LeakLoad_Prism.txt'], createValidationBarsPrism(Exp_leakload_Ctl, NaN, 100*(ca_leak_SR/ca_load_SR), NaN), '\t');

			dlmwrite([output_basename, '_Bar_CaT_Amp_Tecplot.txt'], createValidationBarsTecplot(Exp_Cai_amp_SR, NaN, Cai_amp_SR, NaN), '\t');
			dlmwrite([output_basename, '_Bar_CaT_Amp_Prism.txt'], createValidationBarsPrism(Exp_Cai_amp_SR, NaN, Cai_amp_SR, NaN), '\t');
			
			dlmwrite([output_basename, '_Bar_CaT_TauDecay_Tecplot.txt'], createValidationBarsTecplot(Exp_Cai_tau_SR, NaN, Cai_tau_SR, NaN), '\t');
			dlmwrite([output_basename, '_Bar_CaT_TauDecay_Prism.txt'], createValidationBarsPrism(Exp_Cai_tau_SR, NaN, Cai_tau_SR, NaN), '\t');
			
			dlmwrite([output_basename, '_Bar_Caffeine_CaT_Amp_Tecplot.txt'], createValidationBarsTecplot(1000 * Exp_cCaT_amp_SR, NaN, 1000*(maxval_CaT_SR-minval_CaT_SR), NaN), '\t');
			dlmwrite([output_basename, '_Bar_Caffeine_CaT_Amp_Prism.txt'], createValidationBarsPrism(1000 * Exp_cCaT_amp_SR, NaN, 1000*(maxval_CaT_SR-minval_CaT_SR), NaN), '\t');
		
			dlmwrite([output_basename, '_Bar_Caffeine_CaT_Tau_Tecplot.txt'], createValidationBarsTecplot(Exp_cCaT_tau_SR, NaN, cCaT_tau_SR, NaN), '\t');
			dlmwrite([output_basename, '_Bar_Caffeine_CaT_Tau_Prism.txt'], createValidationBarsPrism(Exp_cCaT_tau_SR, NaN, cCaT_tau_SR, NaN), '\t');
			
			dlmwrite([output_basename, '_Bar_Caffeine_NCX_Peak_Tecplot.txt'], createValidationBarsTecplot(Exp_INCX_peak_SR, NaN, maxval_NCX_SR, NaN), '\t');
			dlmwrite([output_basename, '_Bar_Caffeine_NCX_Peak_Prism.txt'], createValidationBarsPrism(Exp_INCX_peak_SR, NaN, maxval_NCX_SR, NaN), '\t');
			
			dlmwrite([output_basename, '_Bar_Caffeine_NCX_Integral_Tecplot.txt'], createValidationBarsTecplot(Exp_INCX_int_SR, NaN, NCX_int_SR, NaN), '\t');
			dlmwrite([output_basename, '_Bar_Caffeine_NCX_Integral_Prism.txt'], createValidationBarsPrism(Exp_INCX_int_SR, NaN, NCX_int_SR, NaN), '\t');
		end
    
		% Determine error
		% ------------------------------------
		err_cat = [	1.0 .* (Cai_diast_SR - Exp_Cai_diast_SR(1)) ./ Exp_Cai_diast_SR(1); ...
				1.0 .* (Cai_syst_SR - Exp_Cai_syst_SR(1)) ./ Exp_Cai_syst_SR(1); ...				
				1.0 .* (Cai_amp_SR - Exp_Cai_amp_SR(1)) ./ Exp_Cai_amp_SR(1); ...
				1.0 .* (Cai_tau_SR - Exp_Cai_tau_SR(1)) ./ Exp_Cai_tau_SR(1); ...
				1.0 .* ((maxval_CaT_SR-minval_CaT_SR) - Exp_cCaT_amp_SR(1)) ./ Exp_cCaT_amp_SR(1); ...
				1.0 .* (cCaT_tau_SR - Exp_cCaT_tau_SR(1)) ./ Exp_cCaT_tau_SR(1); ...
				1.0 .* (abs(maxval_NCX_SR) - abs(Exp_INCX_peak_SR(1))) ./ abs(Exp_INCX_peak_SR(1)); ...
				1.0 .* (abs(NCX_int_SR) - abs(Exp_INCX_int_SR(1))) ./ abs(Exp_INCX_int_SR(1)); ...	
		]
		
		err_ical = [ 1.0 .* (ICaL_peak_SR - Exp_ICaL_peak_SR(1)) ./ Exp_ICaL_peak_SR(1); ...
				1.0 .* (ICaL_int_SR - Exp_ICaL_int_SR(1)) ./ Exp_ICaL_int_SR(1); ...
				1.0 .* (ICaL_fitparams_SR(2) - Exp_ICaL_Tau_Inact_slow_SR(1)) ./ Exp_ICaL_Tau_Inact_slow_SR(1); ...
				1.0 .* (ICaL_fitparams_SR(1) - Exp_ICaL_Tau_Inact_fast_SR(1)) ./ Exp_ICaL_Tau_Inact_fast_SR(1); ...
				1.0 .* (ICaL_fitparams_SR(3) - Exp_ICaL_Inact_frac_fast_SR(1)) ./ Exp_ICaL_Inact_frac_fast_SR(1); ...
		]
    
end


function err_jphysiol = runCurrentClampJPhysiol(settings, show_it, cparams, loadDataFromFileOnly, output_basename, pAFFactors, ident, simRyR2Gaps, RyR2Heterogeneity, simAxialTubule, simAxialTubuleRyR2Hyperphos)
	% Settings
	% =========================================================================    
  settings.OpenMP = 0;
	settings.levelOfElectroDetail = 51;
	%settings.levelOfElectroDetail = 9;
	settings.run2D = 0;
	settings.DT_MAX = 0.2;
	settings.Output_APD = 'true';		
	settings.Adaptive_dt_Alpha = 0.995;
	settings.skip = 7;
	settings.update = 100000;
	settings.useMarkovICaL = 'true';
	settings.applyVoltageClamp = 'true'; %% Do we still need this?
  settings.IrelB = 0.0;  
    
	% Based on Data Supplement of Voigt, Heijman et al. Circulation 2014:
	settings.RyR_NP_Params = [pAFFactors(7)*0.2,pAFFactors(8),12,0.63,9.0E-4,0.8E-4,pAFFactors(4)*0.0035,6.3,0.0,1.0,1.0,pAFFactors(1) * 0.007,20,-1,-1,-1,-1,-1,-1,-1,1.75,1.0,50,1.5,0.5];		% cparams(16) = 5.75E-4
		
  %%%% These are used for local hyperphosphorylation of RyR2
  RyR_NP_Params = [pAFFactors(7)*0.2,pAFFactors(8),12,0.63,9.0E-4,0.8E-4,pAFFactors(4)*0.0035,6.3,0.0,1.0,1.0,pAFFactors(1) * 0.007,20,-1,-1,-1,-1,-1,-1,-1,1.75,1.0,50,1.5,0.5];
  RyR_P_Params = [pAFFactors(7)*0.2*2,pAFFactors(8)*1.09,12,0.63,9.0E-4,0.8E-4,pAFFactors(4)*0.0035*3,6.3,0.0,1.0,1.0,pAFFactors(1) * 0.007,20,-1,-1,-1,-1,-1,-1,-1,1.75,1.0,50,1.5,0.5];

	if simRyR2Gaps >=1
		settings.numRyRs = (pAFFactors(1) * 198000 * 14) - (198000 * 14 * settings.IrelB);		
    %settings.numRyRs = pAFFactors(1) * 198000 * 19; %settings.numRyRs = pAFFactors(1) * 198000 * 3;
		settings.IupB = 1 - pAFFactors(2) + pAFFactors(2) * 0.4;
		settings.tau_diff_Cai_Domain = pAFFactors(5)*0.6; 	
		settings.tau_diff_Casr_Domain = 5;
		settings.tau_diff_Cai_Segment = pAFFactors(5)*0.6;
		settings.tau_diff_Casr_Segment = 5;		
		settings.tau_diff_Casrs_Segment = 0.22; %0.15;
		settings.tau_diff_Casrs_Domain = 0.16; 
    if pAFFactors(4) <= 2
		  settings.tau_diff_SRS_i = 10; %prev.16
    else 
      settings.tau_diff_SRS_i = 10; %prev.16
    end 
		settings.factDiffJuncSl = 0.01;		settings.factDiffSlMyo = 1.75;
		
		settings.N_Segments = 100; settings.N_CaDomains = 18; 
  else
		settings.numRyRs = (pAFFactors(1) * 198000 * 2.6) - (198000 * 2.6 * settings.IrelB);
		settings.IupB = 1 - pAFFactors(2) + pAFFactors(2) * 0.4;
		settings.tau_diff_Cai_Domain = pAFFactors(5)*0.6; 	
		settings.tau_diff_Casr_Domain = 15;
		settings.tau_diff_Cai_Segment = pAFFactors(5)*0.6;
		settings.tau_diff_Casr_Segment = 15;		
		settings.tau_diff_Casrs_Segment = 0.3; %0.3; %0.15;
		settings.tau_diff_Casrs_Domain = 0.125; 
    if pAFFactors(4) <= 2
		  settings.tau_diff_SRS_i = 12; %prev.16
    else 
      settings.tau_diff_SRS_i = 12; %prev.16
    end 
		settings.factDiffJuncSl = 0.01;		settings.factDiffSlMyo = 1.75;
		
		settings.N_Segments = 50; settings.N_CaDomains = 18; 
  end 
    
	settings.INaCaB = -0.35;
	settings.ICabB = -0.85;
	settings.allowLongitudinalSRSDiff = 'true';
		
	%Based on Voigt, Heijman et al. Circulation 2014 data supplement
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,9.45,70,49.10,10.349,26.553,0.213,10.807,17.5,3,0.2474,13.825,6.3836,0.001,14.9186,1E-6,1.7E-4,1.1E-3,0.1,-1]; % Based on Voigt et al. Circ 2014
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,8.50,70,49.10,10.349,26.553,0.213,10.807,17.5,3,0.2474,13.825,6.3836,0.001,14.9186,1E-6,1.7E-4,1.1E-3,0.1,-1]; % Based on Voigt et al. Circ 2014 with modification in ICaL act (06072018)
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,9.45,70,65,6,12,0.213,10.807,27.5,5,0.2474,43.825,45,0.001,5.0,1E-6,1.7E-4,5.0E-3,0.1,-1];
	settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,13.56,9.45,70,65,6,12,0.213,10.807,27.5,5,0.2474,43.825,45,0.001,5.0,1E-6,1.7E-4,5.0E-3,0.1,-1]; %from Jphysiol	(total = [0]-[24])
	%settings.Params_ICaL = [0.59,0.8,0.052,13,0.132,-4.798,7.5699,70,65,6,13.5578,0.213,10.807,47.1099,5.2958,0.100,103.0244,126.8876,0.001,2.0109,1E-6,2.819E-4,0.00909,0.1,-1]; %Phosphorylated ICaL based on Heijman, 2011	
		
	%settings.IK1B = 1.0; settings.IKrB = 1.0; settings.ITo1B = 1.0; settings.IKsB = 1.0; settings.IKpB = 1.0; settings.IKurB = 1.0;
	%settings.IClbB = 1.0; settings.INaKB = 1.0;	settings.INaB = 1.0;
	settings.Ca_o = 2.0;
	settings.Fluo3 = 0.0; % 0.025;
	settings.cellType = 2;
	%settings.Clamp_Na_i = 8.5;
  if pAFFactors(4) <= 2
	  settings.IK1B = -0.25; %-0.75; %-0.25;
  else 
    settings.IK1B = -0.25;
  end
	settings.ITo1B = 0.25;
	settings.IKpB = 1.0;
	settings.IKurB = 1.0;
  settings.IClbB = 0.0;
	
	settings.Output_Binary_Data = 'true';
	settings.numVarsPerCaDomain = 4;
	if settings.levelOfElectroDetail == 52, settings.numVarsPerCaDomain = 8; end
	settings.startofCaData = 19;
	
	numBeats = 250;
  settings.bcl = 2000;
  settings.applyVoltageClamp = 'false';
  settings.ft = numBeats * settings.bcl;
	settings.startsave = max(0,(numBeats-3)*settings.bcl);
	settings.NUMS2S = 1;
	settings.BCL_S2 = 30000;

	settings.segOutputRes = 1; settings.domOutputRes = 1;
	settings.NumBinaryCols = (settings.startofCaData - 1) + settings.numVarsPerCaDomain * (settings.N_Segments / settings.segOutputRes) * (settings.N_CaDomains / settings.domOutputRes); 
	settings.RyRModelType = 3;
	
	 %ryr_vals= dlmread('intensity3.txt');
    ryr_vals= Diff_distributions(settings.N_Segments, settings.N_CaDomains, RyR2Heterogeneity, simRyR2Gaps);
    ryr_text = sprintf('%f,', ryr_vals'(:));
    dlmwrite([output_basename, '_RyRStructure_Det.txt'], ryr_vals);
    
	if simAxialTubule >= 1
		% ICaL_Distribution  
		ical_structure = ones(settings.N_Segments,settings.N_CaDomains);
		ical_structure(:,2:17)=0;
		%ical_structure(5:24,13)=1;
    %ical_structure(5:24,9)=1;
		
    %random axial tubule --------------------------------------------------
		ical_structure(2,11:12)=1;
		ical_structure(3,9:11)=1;
		ical_structure(2:8,17)=1;
		ical_structure(2,18)=1;
		ical_structure(6:13,10)=1;
		ical_structure(10,12)=1;
		ical_structure(13:28,9)=1;
		ical_structure(14,8)=1;
		ical_structure(13:19,5)=1;
		ical_structure(19:35,4)=1;
		ical_structure(29,8)=1;
		ical_structure(28:29,10)=1;
		ical_structure(29,12)=1;
		ical_structure(24:28,13)=1;
		ical_structure(30:46,11)=1;
		ical_structure(46,9)=1;
		ical_structure(36,5:10)=1;
		ical_structure(34,14)=1;
		ical_structure(46:64,10)=1;
		ical_structure(51,11:13)=1;
		ical_structure(46:47,17)=1;
		ical_structure(56,11)=1;
		ical_structure(57:60,12)=1;
		ical_structure(64:78,9)=1;
		ical_structure(65,11)=1;
		ical_structure(66,12)=1;
		ical_structure(67:69,13)=1;
		ical_structure(69,14)=1;
		ical_structure(79,8)=1;
		ical_structure(79:89,10)=1;
		ical_structure(52,1)=1;
		ical_structure(53:54,2)=1;
		ical_structure(55,3)=1;
		ical_structure(82,18)=1;
		ical_structure(89:91,7)=1;
		ical_structure(91,8)=1;
		ical_structure(55:59,4)=1;
    
		%ical_structure(5:24,5)=1;
		%ical_structure(settings.N_Segments-25:settings.N_Segments-6,5)=1;
		ical_vals = ical_structure;
		%ical_vals = '_icalStructure.txt';
		
	else
		 ical_vals = [1, zeros(1,settings.N_CaDomains-2), 1];
	end 
	ical_text = sprintf('%f,', ical_vals'(:));
    dlmwrite([output_basename, '_icalStructure_Det.txt'], ical_vals);

	if simAxialTubuleRyR2Hyperphos >= 1
		% Local Hyperphosphorylation of RyR2:
		total_RyR_params = [];
		for ii=1:settings.N_Segments
			for jj=1:settings.N_CaDomains
			  %if no axial tubule here:
			  %if (jj >= 11 && jj <=12 && ii==2) || (jj >=9 && jj <=11 && ii==3) || (jj==17 && ii>=2 && ii<=8) || (jj==18 && ii==2) || (jj == 10 && ii>=6 && ii<=13) || (jj == 12 && ii==10) || (jj == 9 && ii>=13 && ii<=28) || (jj == 8 && ii==14) || (jj == 5 && ii>=13 && ii<=19) || (jj == 4 && ii>=19 && ii<=35) || (jj == 8 && ii==29) || (jj == 10 && ii>=28 && ii<=29) || (jj == 12 && ii==29) || (jj == 13 && ii>=24 && ii<=28) || (jj == 11 && ii>=30 && ii<=46) || (jj == 9 && ii==46) || (jj >=5 && jj<=10 && ii==36) || (jj == 14 && ii==34) || (jj == 10 && ii>=46 && ii<=64) || (jj >= 11 && jj <=13 && ii==51) || (jj == 17 && ii>=46 && ii<=47) || (jj == 11 && ii==56) || (jj == 12 && ii>=57 && ii<=60) || (jj == 9 && ii>=64 && ii<=78) || (jj == 11 && ii==65) || (jj == 12 && ii==66) || (jj == 13 && ii>=67 && ii<=69) || (jj == 14 && ii==69) || (jj == 8 && ii==79) || (jj == 10 && ii>=79 && ii<=89) || (jj == 1 && ii==52) || (jj == 2 && ii>=53 && ii<=54) || (jj == 3 && ii==55) || (jj == 18 && ii==82) || (jj == 7 && ii>=89 && ii<=91) || (jj == 8 && ii==91) || (jj == 4 && ii>=55 && ii<=59)    
			  %if (jj == 13 && ii>=5 && ii<=24) || (jj == 5 && ii>=settings.N_Segments-25 && ii<=settings.N_Segments-6)
			  if jj > 1 && jj < 18 && ical_structure(ii,jj) == 1
				total_RyR_params = [total_RyR_params, RyR_P_Params];
			  else
				total_RyR_params = [total_RyR_params, RyR_NP_Params];
			  end
			end
		end
		settings.RyR_NP_Params = total_RyR_params;
	end
	
	settings.hideOutput = 'false';
	settings.Other = strvcat(['SERCA_Kmr = ', num2str(pAFFactors(6)*1.0)], ...
		['SERCA_Kmf_NP = ', num2str(pAFFactors(3)*6.25E-4)], ...
		'Update_X0 = false', ...
		'Load_X0 = false', ...
		['X0_File = X0_100_seg_homogenous_incr_ryr_po_100pct.txt'], ...			
		'VjuncFact = 0.001', ...
		['Ca_Buff_Factor = 1.0'], ...
		['ICaLDistribution = ', ical_text(1:end-1)], ...
		%['ICaLDistribution = 1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1'], ...
		['RyRDistribution =  ', ryr_text(1:end-1)], ...
		['NCXDistribution =  1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1'], ...
		['SERCADistribution =1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1'] ...
	);

  %settings.Update_X0 = 'true';
  %settings.Load_X0 = 'false';
  %settings.X0_File = 'X0_increase_serca2a_min25pctkmf.txt';
  
	if loadDataFromFileOnly ~= 1
		data_SR = HAMM(settings, 'Parameters_compareRyRSERCAWaves_Stoch.txt', [output_basename, '_AllData']);
	else
		data_SR = loadTissueData([output_basename, '_AllData_2000_ms.txt'], settings.NumBinaryCols);
	end
		
	settings.N_Segments = settings.N_Segments / settings.segOutputRes; 
	settings.N_CaDomains = settings.N_CaDomains / settings.domOutputRes; 
		
		% Analysis
		% ------------------------------------
			[val, ind_start_normal] = min(abs(data_SR(:,1) - 3*settings.bcl));
			[val, ind_end_normal] = min(abs(data_SR(:,1) - (3*settings.bcl+settings.bcl)));
			%[val, ind_start_tetracaine] = min(abs(data_SR(:,1) - (3*settings.bcl+settings.bcl+10000)));
			%[val, ind_end_tetracaine] = min(abs(data_SR(:,1) - (3*2064+2064+15000)));
			
			diast_level_normal_SR = min(data_SR(ind_start_normal:ind_end_normal,3))
			syst_level_normal_SR = max(data_SR(ind_start_normal:ind_end_normal,3))
			%diast_level_tetracaine_SR = min(data_SR(ind_start_tetracaine:ind_end_tetracaine,3))
			%syst_level_caffeine_SR = max(data_SR(ind_end_tetracaine:end,3))
			%ca_leak_SR = diast_level_normal_SR - diast_level_tetracaine_SR
			%ca_load_SR = syst_level_caffeine_SR - diast_level_normal_SR
			
			%Exp_leak_Ctl = [31.62, 6.119]; %Voigt et al. Circulation 2012: [59.61945032, 7.293868922];
			%Exp_leak_pAF = [59.55, 8.467]; Exp_leak_cAF = [92.2832981, 13.3192389];
	
			%Exp_leakload_Ctl = [2.747, 0.6679]; %Voigt et al. Circulation 2012: [4.451476793, 0.548523207];
			%Exp_leakload_pAF = [2.590, 0.5955]; Exp_leakload_cAF = [7.679324895, 1.075949367];
		
			%mean_Im_diastole = mean(data_SR(ind_end_normal:ind_start_tetracaine,9))
			%mean_Im_diastole = mean(data_SR(ind_end_normal:ind_start_tetracaine-10,9))
			%mean_Im_diastole = mean(data_SR(ind_end_normal:ind_start_tetracaine,9)-max(data_SR(ind_end_normal:ind_start_tetracaine,9)))
			%mean_Im_diastole = mean(data_SR(ind_end_normal:ind_start_tetracaine-10,9)-max(data_SR(ind_end_normal:ind_start_tetracaine-10,9)))
		
			indices_membr = settings.startofCaData+(0:1:(settings.N_Segments-1)).*(settings.numVarsPerCaDomain*settings.N_CaDomains)+(2 - 1)*settings.numVarsPerCaDomain;
			indices_centr = settings.startofCaData+(0:1:(settings.N_Segments-1)).*(settings.numVarsPerCaDomain*settings.N_CaDomains)+(9 - 1)*settings.numVarsPerCaDomain;	
			Ca_membr = 1E6 .* mean(data_SR(ind_start_normal:ind_end_normal,indices_membr),2); Ca_membr_amp = max(Ca_membr) - min(Ca_membr)
			Ca_centr = 1E6 .* mean(data_SR(ind_start_normal:ind_end_normal,indices_centr),2); Ca_centr_amp = max(Ca_centr) - min(Ca_centr)
			[val, ind_membr_max] = max(Ca_membr);
			[val, ind_centr_max] = max(Ca_centr);
			CaT_ratio = Ca_centr_amp ./ Ca_membr_amp
			CaT_delay = data_SR(ind_start_normal+ind_centr_max,1) - data_SR(ind_start_normal+ind_membr_max,1)
			
			Exp_SS_CC_Amp = [0.929, 0.059];
			Exp_SS_CC_Delay = [50, 10];
			
			%xlimplot = [numBeats*settings.bcl + 1500, numBeats*settings.bcl + settings.bcl + 21500];
      xlimplot = [3*settings.bcl + 1500, 3*settings.bcl + settings.bcl + 28000];
			[val, ind_s] = min(abs(data_SR(:,1) - xlimplot(1)))
			[val, ind_e] = min(abs(data_SR(:,1) - xlimplot(2)))
			data_new_SR = data_SR(ind_s:ind_e,:);
			data_new_SR(:,3) = data_new_SR(:,3) .* 1E6;
			opt_struct.skip = 1; opt_struct.xlabel = 'time (ms)'; opt_struct.ylabel = strvcat('Vm (mV)', 'ICaL (pA/pF)', 'Cai (nmol/l)', 'Casr (mmol/l)');
			if xlimplot(2) - xlimplot(1) < 500, opt_struct.delay = 0.1; end
			
			Ca_i_data_SR = 1E3 * getDataMatrix(data_new_SR, settings.startofCaData + 3, settings.N_Segments, settings.N_CaDomains, settings.numVarsPerCaDomain);
			options.threshold = 3;
			size(Ca_i_data_SR)
			waves = detectCalciumWaves(data_new_SR(:,1) - data_new_SR(1,1), Ca_i_data_SR, settings, options)
			waves_text = sprintf('%f,', waves'(:));
	    dlmwrite([output_basename, '_Waves.txt'], waves);
      
			if ~isempty(waves)
				sizeperunit = 1 * settings.domOutputRes * 2 * settings.segOutputRes % in um^2
				avg_size = sizeperunit * mean(waves(:,4))
				rel_avg_size = sizeperunit * mean(waves(:,4)) ./ (100 * 18) % fraction of recorded area
				numwaves = sum(waves(:,4) > 2);
				numsparks = size(waves,1) - numwaves;
				numsprkpsec = numsparks ./ ((xlimplot(2) - xlimplot(1)) / 1000) ./ (0.1 * 18) % Based on 18 'parallel' linescans of 0.1 mm length
				numwavpsec = numwaves ./ ((xlimplot(2) - xlimplot(1)) / 1000) ./ (0.1 * 18) % Based on 18 'parallel' linescans of 0.1 mm length
			  endtimewaves= waves(:,1) + sizeperunit*(waves(:,4));

      else
				numwaves = 0;
        avg_size = 0;
				rel_avg_size = 0;
				numsprkpsec = 0;
				numwavpsec = 0;
			end
			
      [val, ind2] = min(abs(data_SR(:,1) - numBeats * settings.bcl));
			CaT_Amp = max(data_SR(1:ind2,3)) - min(data_SR(1:ind2,3))
			SR_content = max(data_SR(ind_e:end,3)) - min(data_SR(ind_e:end,3))
			max_DAD_amp = max(data_SR(ind_s:ind_e,2)) - min(data_SR(ind_s:ind_e,2))
			
			Exp_spark_incidence_Ctl = [2.8, 0.8];
			Exp_wave_incidence_Ctl = [1.1, 0.3];
			Exp_SS_CC_Amp = [0.929, 0.059];
			Exp_SS_CC_Delay = [50, 10];
		
		% Visualization
		% ------------------------------------
		if show_it >= 1
			xlimplot = [0 90000];
			[val, ind_s] = min(abs(data_SR(:,1) - xlimplot(1)));
			[val, ind_e] = min(abs(data_SR(:,1) - xlimplot(2)));
			
			data_new_SR = data_SR(ind_s:ind_e,:);
			data_new_SR(:,3) = data_new_SR(:,3) .* 1E3;
			opt_struct.skip = 4; opt_struct.xlabel = 'time (ms)'; opt_struct.ylabel = strvcat('Vm (mV)', '[Ca2+]i (nmol/l)');
			opt_struct.max = 600; opt_struct.min = 100;
			opt_struct.trace_size = [1;3];
			opt_struct.ylim = [-90, 50; 0, 2000]; %[-90, 50; 0, 800];
			opt_struct.linecolor = 'k';
			opt_struct.font_size = 14;
			opt_struct.ylabel_pos = -4000;
			opt_struct.plotfunc = @drawProtocol;
			
			%if createMovie == 1
			%	opt_struct.movie_file = [output_basename, '_Movie.avi'];
			%end
			
			if xlimplot(2) - xlimplot(1) < 500, opt_struct.delay = 0.1; end
			Ca_i_data_SR = 1E6 * getDataMatrix(data_new_SR, settings.startofCaData, settings.N_Segments, settings.N_CaDomains, settings.numVarsPerCaDomain);
			%animateSimulation(data_new_SR(:,1) - data_new_SR(1,1), [data_new_SR(:,[2,9,3])], NaN, Ca_i_data_SR, settings.N_Segments, settings.N_CaDomains, opt_struct)	
			animateSimulation(data_new_SR(:,1) - data_new_SR(1,1), [data_new_SR(:,[2,9,3])], [1;3], Ca_i_data_SR, settings.N_Segments, settings.N_CaDomains, opt_struct)	
			data_new_SR(:,3) = data_new_SR(:,3) .* 1E3;
			
			figure
				set(gcf, 'Position', [200,100,1000,800]);
				subplot(6,5,[1,2,3,4,5]); 
					plot((data_SR(ind_s,1):1:data_SR(ind_e,1))', interp1(data_SR(:,1),data_SR(:,4),(data_SR(ind_s,1):1:data_SR(ind_e,1))'), 'k', 'LineWidth', 2);
					colorbar; grid on; axis tight; axis off;
          ylabel('Vm (mV)');
          csvwrite('Vm_100segments_gaps02_exp_axial.txt', [data_SR(:,1), data_SR(:,2)]);  

			    csvwrite('Cai_100segments_gaps02_exp_axial.txt', [data_SR(:,1), data_SR(:,3)]);

          csvwrite('SR_100segments_gaps02_exp_axial.txt', [data_SR(:,1), data_SR(:,4)]);          
          
        %Longitudinal wave
        subplot(6,5,(6:15))
          rowindex = floor(0.5* settings.N_Segments / settings.segOutputRes);
          Ca_i_data_SR_L = interp1(data_new_SR(:,1), Ca_i_data_SR(:,(9:settings.N_CaDomains:end)), (data_SR(ind_s,1):4:data_SR(ind_e,1))');
          imagesc((data_SR(ind_s,1):4:data_SR(ind_e,1))'-data_SR(ind_s,1),(1:settings.N_Segments)', Ca_i_data_SR_L'); colorbar; ylabel(colorbar, '[Ca2+] (nmol/l)'); grid off; colormap('jet');
          xlabel('Time [ms]');
          ylabel('Segments');          
          
				subplot(6,5,(16:25))
					rowindex = floor(25 / settings.segOutputRes);
					Ca_i_data_SR = interp1(data_new_SR(:,1), Ca_i_data_SR(:,((rowindex-1)*settings.N_CaDomains+1):(rowindex*settings.N_CaDomains)), (data_SR(ind_s,1):4:data_SR(ind_e,1))');
					imagesc((data_SR(ind_s,1):4:data_SR(ind_e,1))'-data_SR(ind_s,1),(1:settings.N_CaDomains)',Ca_i_data_SR'); colorbar; grid off; colormap('jet');
				
        %subplot(3,4,[5,9]); bar(avg_size); ylabel('avg. wave size (\mum^2)');
        
				%subplot(6,5,16:20); 
				%	plot((data_SR(ind_s,1):1:data_SR(ind_e,1))', interp1(data_SR(:,1),data_SR(:,9),(data_SR(ind_s,1):1:data_SR(ind_e,1))'), 'k', 'LineWidth', 2);
				%	colorbar; grid on; axis tight; axis off;
				%subplot(6,5,5+[16,21]); bar(100 * rel_avg_size); ylabel('rel wave size (% of recorded area)'); ylim([0, 100]);
				%subplot(6,5,5+[17,18,22,23]); 
				%	h = bar([Exp_spark_incidence_Ctl(1), numsprkpsec; Exp_wave_incidence_Ctl(1), numwavpsec]);
				%	set (h(1), 'facecolor', 'w', 'LineWidth', 2);
				%	set (h(2), 'facecolor', 'b', 'LineWidth', 2);
				%	ylabel('incidence (ms^{-1})');
				%subplot(6,5,5+[19,24]); plotSRAFBars(Exp_SS_CC_Amp, NaN, CaT_ratio, NaN, 'CC / SS CaT Ratio');
				%subplot(6,5,5+[20,25]); plotSRAFBars(Exp_SS_CC_Delay, NaN, CaT_delay, NaN, 'CC - SS CaT Delay'); 

      %Transversal wave
      data_new_SR(:,3) = data_new_SR(:,3) .* 1E3;
			Ca_i_data_SR = 1E6 * getDataMatrix(data_new_SR, settings.startofCaData, settings.N_Segments, settings.N_CaDomains, settings.numVarsPerCaDomain);

			figure
				set(gcf, 'Position', [200,100,1000,800]);
				subplot(6,5,[1,2,3,4,5]); 
        S=1;
        E=500;
        xlim([S E]);
          plot((data_SR(ind_s,1):1:data_SR(ind_e,1))', interp1(data_SR(:,1),data_SR(:,2),(data_SR(ind_s,1):1:data_SR(ind_e,1))'), 'k', 'LineWidth', 2);
          colorbar; grid off; axis tight; 
          %ylim([-80 40]);
          set(gca, 'YTick', [])
          %xlim([1500 35000]);
          xlim([S E]);
          set(gca, 'XTick', [])
          ylabel('Vm (mV)');
        subplot(6,5,6:15); 
				  plot((data_SR(ind_s,1):1:data_SR(ind_e,1))', interp1(data_SR(:,1),data_SR(:,3),(data_SR(ind_s,1):1:data_SR(ind_e,1))'), 'k', 'LineWidth', 2);
					colorbar; grid off; axis tight; 
          dlmwrite('intracellular calcium.txt',[(data_SR(:,1)-2000) (data_SR(:,3)*1000)]);
          ylim([0.2 0.4]);
          set(gca, 'YTick', [])
          xlim([S E]);
          %xlim([1500 35000]);
          set(gca, 'XTick', [])
          ylabel('[Ca2+]i(nmol/l)');
        subplot(6,5,(16:25))
        rowindex = 1+floor(0.15 * settings.N_Segments / settings.segOutputRes);
        Ca_i_data_SR_T = interp1(data_new_SR(:,1), Ca_i_data_SR(:,((rowindex+1-1)*settings.N_CaDomains+1):((rowindex+1)*settings.N_CaDomains)), (data_SR(ind_s,1):1:data_SR(ind_e,1))'); %ryr
        Ca_i_data_SR_T_Alt = interp1(data_new_SR(:,1), Ca_i_data_SR(:,((rowindex-1)*settings.N_CaDomains+1):((rowindex)*settings.N_CaDomains)), (data_SR(ind_s,1):1:data_SR(ind_e,1))'); %gap
        time_local_CaT = (data_SR(ind_s,1):1:data_SR(ind_e,1))'-data_SR(ind_s,1);
        imagesc(time_local_CaT,(1:settings.N_CaDomains)',Ca_i_data_SR_T'); colorbar; grid off; colormap('jet');
				%xlim([2000 2500]);
        xlim([S E]);
        %xlim([1 1000]);
        %set(gca, 'XTick', [])
        set(gca, 'YTick', [])
        
        avg_local_CaT = mean(Ca_i_data_SR_T')';
        avg_local_CaT_Alt = mean(Ca_i_data_SR_T_Alt')';
        [val, ind_s] = min(abs(time_local_CaT - 2000));
        [val, ind_e] = min(abs(time_local_CaT - 4000));
  
        [val, ind] = max(avg_local_CaT_Alt(ind_s:ind_e));
        local_time_to_peak_gap = time_local_CaT(ind+ind_s) - time_local_CaT(ind_s)
        
        [val, ind] = max(avg_local_CaT(ind_s:ind_e));
        local_time_to_peak_ryr = time_local_CaT(ind+ind_s) - time_local_CaT(ind_s)
        
        average_ttp = (local_time_to_peak_gap + local_time_to_peak_ryr) / 2
        
        %figure;
        %hold on;
        %plot(time_local_CaT(ind_s:ind_e), avg_local_CaT(ind_s:ind_e));
        
        [val, ind_s] = min(abs(data_SR(:,1) - 2000));
        [val, ind_e] = min(abs(data_SR(:,1) - 4000));
        
        %plot(data_SR(ind_s:ind_e,1), 1E3.*data_SR(ind_s:ind_e,3), '-r');

        %figure;
        % plot(data_SR(1:end,1), 1E3.*data_SR(1:end,3), '-r', time_local_CaT(1:end), avg_local_CaT(1:end),'-b', time_local_CaT(1:end), avg_local_CaT_Alt(1:end),'-k');
        
        min_cai= min(data_SR(ind_s:ind_e,3))
        [max_cai, maxind]= max(data_SR(ind_s:ind_e,3))
        ca_i_amp= max_cai-min_cai
        %[maxval, maxind] = max(data_SR(ind_s:ind_e,3));
        global_time_to_peak= data_SR(ind_s+maxind,1) - data_SR(ind_s,1)
        
        Velocity(0)
    print('figure', '-dpdf')
        
		end
		
		% Output
		% ===================================================	
		if ~isempty(output_basename)
				dlmwrite([output_basename, '_Traces.txt'], data_SR(:,1:9), '\t');
		    %dlmwrite([outputbasename, '_Traces.txt'], data_SR(:,1:(settings.startofCaData-1)), '\t');
		    createMeshDataFile([(data_SR(ind_s,1):4:data_SR(ind_e,1))'-data_SR(ind_s,1), 1E6 .* Ca_i_data_SR], [output_basename, '_Ca_LineScan.txt']);
		    fid = fopen([output_basename, '_Summary.txt'],'w');    
		    	fprintf(fid,'numwaves = %f\n', numwaves);
		    	fprintf(fid,'numwavpsec = %f\n', numwavpsec);
		    	fprintf(fid,'avg_size = %f\n', rel_avg_size);
		    	fprintf(fid,'CaT Amp = %f\n', CaT_Amp);
		    	fprintf(fid,'SR Content = %f\n', SR_content);
		    	fprintf(fid,'max_DAD_amp = %f\n', max_DAD_amp);
		    fclose(fid);
					
				rowindex = 12;
				xlimplot = [0, 350000];
				[val, ind_s] = min(abs(data_SR(:,1) - xlimplot(1)));
				[val, ind_e] = min(abs(data_SR(:,1) - xlimplot(2)));
				Ca_i_data_SR = getDataMatrix(data_SR, settings.startofCaData, settings.N_Segments, settings.N_CaDomains, settings.numVarsPerCaDomain);
				Ca_i_data_SR = interp1(data_SR(:,1), Ca_i_data_SR(:,((rowindex-1)*settings.N_CaDomains+1):(rowindex*settings.N_CaDomains)), (data_SR(ind_s,1):2:data_SR(ind_e,1))');
				createMeshDataFile([(data_SR(ind_s,1):2:data_SR(ind_e,1))'-data_SR(ind_s,1), 1E6 .* Ca_i_data_SR], [output_basename, 'Ca_LineScan.txt']);
				
				ttt =(data_SR(ind_s,1):2:data_SR(ind_e,1))';
				Ca_i_data_avg = mean(1E6 .* Ca_i_data_SR,2);		
				[val, ind_end_normal] = min(abs(ttt - (3*2064+2064)));
				[val, ind_start_tetracaine] = min(abs(ttt - (3*2064+2064+10000)));
				Ampl_Ca_i_Avg = mean(Ca_i_data_avg(ind_end_normal:ind_start_tetracaine) - min(Ca_i_data_avg(ind_end_normal:ind_start_tetracaine)));
				
				%figure;
				%	plot(ttt(ind_end_normal:ind_start_tetracaine), Ca_i_data_avg(ind_end_normal:ind_start_tetracaine), '-k', 'LineWidth', 2)
				
				dlmwrite([output_basename, '_Bar_LineScan_SCaE_Ca_Ampl_Tecplot.txt'], createValidationBarsTecplot([0,0], NaN, Ampl_Ca_i_Avg, NaN), '\t');
				dlmwrite([output_basename, '_Bar_LineScan_SCaE_Ca_Ampl_Prism.txt'], createValidationBarsPrism([0,0], NaN, Ampl_Ca_i_Avg, NaN), '\t');
				
				%dlmwrite([output_basename, '_Bar_SR_Ca_Leak_Tecplot.txt'], createValidationBarsTecplot(Exp_leak_Ctl, NaN, 1000*ca_leak_SR, NaN), '\t');
				%dlmwrite([output_basename, '_Bar_SR_Ca_Leak_Prism.txt'], createValidationBarsPrism(Exp_leak_Ctl, NaN, 1000*ca_leak_SR, NaN), '\t');
				
				%dlmwrite([output_basename, '_Bar_SR_Ca_LeakLoad_Tecplot.txt'], createValidationBarsTecplot(Exp_leakload_Ctl, NaN, 100*(ca_leak_SR/ca_load_SR), NaN), '\t');
				%dlmwrite([output_basename, '_Bar_SR_Ca_LeakLoad_Prism.txt'], createValidationBarsPrism(Exp_leakload_Ctl, NaN, 100*(ca_leak_SR/ca_load_SR), NaN), '\t');
				
				%dlmwrite([output_basename, '_Bar_SR_Ca_Load_Tecplot.txt'], createValidationBarsTecplot(Exp_leakload_Ctl, NaN, ca_load_SR, NaN), '\t');
				%dlmwrite([output_basename, '_Bar_SR_Ca_Load_Prism.txt'], createValidationBarsPrism(Exp_leakload_Ctl, NaN, ca_load_SR, NaN), '\t');
				
				dlmwrite([output_basename, '_Bar_Wave_Size_Tecplot.txt'], createValidationBarsTecplot([0,0], NaN, avg_size, NaN), '\t');
				dlmwrite([output_basename, '_Bar_Wave_Size_Prism.txt'], createValidationBarsPrism([0,0], NaN, avg_size, NaN), '\t');
				
				dlmwrite([output_basename, '_Bar_Relative_Wave_Size_Tecplot.txt'], createValidationBarsTecplot([0,0], NaN, 100*rel_avg_size, NaN), '\t');
				dlmwrite([output_basename, '_Bar_Relative_Wave_Size_Prism.txt'], createValidationBarsPrism([0,0], NaN, 100*rel_avg_size, NaN), '\t');
				
				dlmwrite([output_basename, '_Bar_Wave_Incidence_Tecplot.txt'], createValidationBarsTecplot(Exp_wave_incidence_Ctl, NaN, numwavpsec, NaN), '\t');
				dlmwrite([output_basename, '_Bar_Wave_Incidence_Prism.txt'], createValidationBarsPrism(Exp_wave_incidence_Ctl, NaN, numwavpsec, NaN), '\t');
				
				dlmwrite([output_basename, '_Bar_Spark_Incidence_Tecplot.txt'], createValidationBarsTecplot(Exp_spark_incidence_Ctl, NaN, numsprkpsec, NaN), '\t');
				dlmwrite([output_basename, '_Bar_Spark_Incidence_Prism.txt'], createValidationBarsPrism(Exp_spark_incidence_Ctl, NaN, numsprkpsec, NaN), '\t');
				
				%dlmwrite([output_basename, '_Bar_Avg_Im_Diastole_Tecplot.txt'], createValidationBarsTecplot([0,0], NaN, mean_Im_diastole, NaN), '\t');
				%dlmwrite([output_basename, '_Bar_Avg_Im_Diastole_Prism.txt'], createValidationBarsPrism([0,0], NaN, mean_Im_diastole, NaN), '\t');
		end	
end

function drawProtocol(indx, axis_handle, plot_handle)
	x_offset = 2000;
	fons = 14;
	if indx == 1
		numBeats = 3;
		rectangle('Position',[0, 20, numBeats*2064+2064+10000-x_offset, 20], 'EdgeColor', 'k', 'FaceColor', 'w');
		text(0.5*(numBeats*2064+2064+10000-x_offset),33,'Normal Tyrode','HorizontalAlignment','center','FontSize',fons);
		rectangle('Position',[numBeats*2064+2064+10000-x_offset, 20, 10000, 20], 'EdgeColor', 'k', 'FaceColor', 'k');
		text(numBeats*2064+2064+10000+4500-x_offset,33,'0 Na+, 0 Ca2+', 'HorizontalAlignment', 'center', 'Color', 'w','FontSize',fons);
		
		%settings.RyRPo_times = [numBeats*2064+2064+10000, numBeats*2064+2064+15000, numBeats*2064+2064+16000, numBeats*2064+2064+17000, numBeats*2064+2064+17005, numBeats*2064+2064+17010, numBeats*2064+2064+17015, numBeats*2064+2064+17020, numBeats*2064+2064+20000, 1E6];
		rectangle('Position', [numBeats*2064+2064+10000-x_offset, -50, 5000, 1], 'EdgeColor', 'k');
		text(numBeats*2064+2064+10000+2500-x_offset,-37, 'Tetracaine', 'HorizontalAlignment', 'center','FontSize',fons);
		
		rectangle('Position', [numBeats*2064+2064+17000-x_offset, -20, 3000, 1], 'EdgeColor', 'k');
		text(numBeats*2064+2064+17000-1000-x_offset,-7, 'Caffeine -->', 'HorizontalAlignment', 'center','FontSize',fons);
	end
end

function plotSRAFBars(Exp_Data_SR, Exp_Data_AF, Mdl_SR, Mdl_AF, title_s)
	hold on;
	if isnan(Exp_Data_AF) || isnan(Mdl_AF)
		h1 = bar(1, [Exp_Data_SR(1)], 0.4, 'facecolor', 'w', 'LineWidth', 2);
		h2 = bar(2, Mdl_SR, 0.4, 'facecolor', 'k', 'LineWidth', 2);		
		if ~isempty(title_s)
			title(title_s);
		end
		set(gca, 'xtick', [0.5; 1; 1.5; 2; 2.5]);
		set(gca, 'xticklabel', strvcat(' ', 'Exp.', ' ', 'Mdl.', ' '));
		herr = errorbar(1, [Exp_Data_SR(:,1)], [Exp_Data_SR(:,2)], '~');
		set(herr, 'linestyle', 'none', 'Color', 'black', 'MarkerSize', 10);

	else
		h = bar([Exp_Data_SR(1), Mdl_SR; Exp_Data_AF(1), Mdl_AF]);		
		if ~isempty(title_s)
			title(title_s);
		end
		set(gca, 'xtick', [0.5; 1; 1.5; 2; 2.5]);
		set(gca, 'xticklabel', strvcat(' ', 'SR', ' ', 'AF', ' '));
		set (h(1), 'facecolor', 'w', 'LineWidth', 2);
		set (h(2), 'facecolor', 'k', 'LineWidth', 2);
		herr = errorbar(get(h(1), 'xdata')-0.2, [Exp_Data_SR(:,1); Exp_Data_AF(:,1)], [Exp_Data_SR(:,2); Exp_Data_AF(:,2)], '~');
		set(herr, 'linestyle', 'none', 'Color', 'black', 'MarkerSize', 10);

	end
	
end

function omtrx = createValidationBarsTecplot(Exp_Data_SR, Exp_Data_AF, Mdl_SR, Mdl_AF)
	if isnan(Exp_Data_AF) || isempty(Exp_Data_AF)
		omtrx = [1, Exp_Data_SR, 0, 0; 2, 0, 0, Mdl_SR, 0];
	else
		omtrx = [1, Exp_Data_SR, 0, 0; 2, 0, 0, Mdl_SR, 0; 4, Exp_Data_AF, 0, 0; 5, 0, 0, Mdl_AF, 0];
	end
end

function omtrx = createValidationBarsPrism(Exp_Data_SR, Exp_Data_AF, Mdl_SR, Mdl_AF)
	if isnan(Exp_Data_AF) || isempty(Exp_Data_AF)
		omtrx = [Exp_Data_SR, 1, Mdl_SR, 0, 1];
	else
		omtrx = [Exp_Data_SR, 1, Mdl_SR, 0, 1; Exp_Data_AF, 1, Mdl_AF, 0, 1];
	end
end