#include <math.h>
#include <stdio.h>
#include <iostream>
#include "genpurpose.h"
#include "constants.h"
#include "settings.h"
#include "MersenneTwister.h"

class HAMM
{
	public:
		bool showlog;
		double caiont, kiont, naiont, clont;
		
		double Vm;
		
		double Na_i, Na_srs, Na_sl, K_i;
		double Ca_junc, Ca_sl, Ca_sr, Ca_srs, Ca_i;
		
		double INa, INa_junc, INa_sl, INaL, INaL_junc, INaL_sl, INaK, INaK_junc, INaK_sl, INab, INab_junc, INab_sl, IKr, IKs, IK1, IKp, IKur, ITo1, IClb, IKAch;
		double ICaL_junc, ICaL_sl, ICaL_K, ICaL_Na_junc, ICaL_Na_sl, INaCa_junc, INaCa_sl, IpCa_junc, IpCa_sl, IClCa_junc, IClCa_sl, ICab_junc, ICab_sl; // Whole-cell equivalents of CaRU currents
		double Jrel, Jleak, Jup, Buff_Ca_EGTA_cyt;
		double Istim; 
		
		double ECl, ENa_junc;
		
		Constants* data;
		Settings* settings;

		struct CaDomain
		{
			double RyRState_NP[8];
			double RyR_SEP;
			double RyR_Integr;
			double Ca_i, Ca_sr, Ca_srs;
			double Na_i, Na_srs, Buff_Na_junc;
			double Buff_Ca_TnCL, Buff_Ca_TnCHc, Buff_Ca_TnCHm, Buff_Ca_CaM, Buff_Ca_Myosin_ca, Buff_Ca_Myosin_mg, Buff_Ca_SRB, Buff_Ca_CSQN;
			double Buff_Ca_SLLsrs, Buff_Ca_SLHsrs;
			double Buff_Ca_EGTA_cyt;
			
			double ICaLState_NP[8];			
			double INaCa_junc, INaCa_sl, ICaL_junc, ICaL_sl, ICaL, ICaL_Na, ICaL_Na_junc, ICaL_Na_sl, ICaL_K, ICaL_tot;
			
			double Jrel, Jleak, Jup;
			double J_diff_i_subcell;
		};
		
		struct MembrDomain
		{
			double Ca_junc, Ca_sl;
			double Na_sl;
			
			double m_Na, h_Na, j_Na, mL_Na, hL_Na, xr, to_a, to_if, kur_a, kur_i, xs_NP, xs_P;
			
			double Buff_Ca_SLLj, Buff_Ca_SLLsl, Buff_Ca_SLHj, Buff_Ca_SLHsl;
			double Buff_Na_sl, Buff_Ca_EGTA_sl;
			
			double INa_junc, INa_sl, INaL_junc, INaL_sl, INaK_junc, INaK_sl, INab_junc, INab_sl, IKr, IKs, IK1, IKp, IKur, ITo1, IClb, IKAch; //, ISK;
			
			double IClCa_junc, IClCa_sl, INaCa_junc, INaCa_sl, IpCa_junc, IpCa_sl;
			double ICab_junc, ICab_sl;
		};
		
		struct Segment
		{
			MembrDomain membrane[2];
			CaDomain* caUnits;
			double K_i;
		};
		
		Segment* segments;
		
		double** Copy_Ca_i;
		double** Copy_Ca_sr;
		double** Copy_Ca_srs;
		double** Copy_Ca_sl;
		double** Copy_Na_sl;
		double** Copy_Na_i;
		double** Copy_Na_srs;
		
		double vrest, vmax;
		double tmaxdvdt, valmaxdvdt, Max_State_Change;
		double APD, CaT_Min, CaT_Max;

		void init(Constants *idata, Settings *isettings);
		void update(double t, double dt);
		void update(int cellnumber, double t, double dt, double vleft, double vright);
		void outputData(FILE* outputfile, int levelOfElectroDetail, int levelOfSignalingDetail, bool binaryOutput);
		void update(int cellnumber, double t, double dt, double vleft, double vright, double vup, double vdown);
		HAMM* clone();

		void loadX0Data(double* x0data, int x0length);
		int lengthMinimumStateVector();
		
	private:
		MTRand rvgen;	
		
		double xsss_NP, tauxs_NP, xsss_P, tauxs_P, sigma, fnak; // Helper variables so that these only have to be calculated once per timestep
		double xtoss, tauxtof, ytoss, tauytof, xkurss, tauxkur, ykurss, tauykur, gkr, ikr_r;
		
		void processX0DataForMarkovModel(double* pX0Data, int sindex, double* MM, int num_states, int model_type, int num_channels);
		void processX0DataForMarkovModel(double* pX0Data, int sindex, int* MM, int num_states, int model_type, int num_channels);
		
		void updateCaDomain(double dt, double t, int segment, int ca_domain, double Ca_i_L_Domain, double Ca_i_R_Domain, double Ca_i_U_Segment, double Ca_i_D_Segment, double Ca_sr_L_Domain, double Ca_sr_R_Domain, double Ca_sr_U_Segment, double Ca_sr_D_Segment, double Ca_srs_L_Domain, double Ca_srs_R_Domain, double Ca_srs_U_Segment, double Ca_srs_D_Segment);
		void updateNaDomain(double dt, double t, int segment, int ca_domain, double Na_i_L_Domain, double Na_i_R_Domain, double Na_i_U_Segment, double Na_i_D_Segment, double Na_srs_L_Domain, double Na_srs_R_Domain, double Na_srs_U_Segment, double Na_srs_D_Segment);
		
		void Ion_HH_Na(int seg_i, int mem_i, double dt);		
		void Ion_HH_NaL(int seg_i, int mem_i, double dt);
		void Ion_HH_NaCa(int seg_i, int cadom_i, double dt);
		void Ion_HH_NaK(int seg_i, int mem_i, double dt);
		void Ion_HH_Nab(int seg_i, int mem_i, double dt);
		
		void Ion_HH_K1(int seg_i, int mem_i, double dt);
		void Ion_HH_Kp(int seg_i, int mem_i, double dt);
		void Ion_HH_KAch(int seg_i, int mem_i, double dt);
		void Ion_HH_Kur(int seg_i, int mem_i, double dt);
		void Ion_HH_Kr(int seg_i, int mem_i, double dt);
		void Ion_HH_To1(int seg_i, int mem_i, double dt);
		void Ion_HH_Cl_Background(int seg_i, int mem_i, double dt);
		
		void Ion_HH_Cl(int seg_i, int mem_i, double dt);
		void Ion_HH_Ks(int seg_i, int mem_i, double dt);
		void Ion_Markov_CaL(int seg_i, int mem_i, double dt);
		void Ion_Ca_Handling(int seg_i, int mem_i, double dt);
		void Ca_CICR(int seg_i, int cadom_i, double dt, double t);
		void Ca_CICR_Stoch(int seg_i, int cadom_i, double dt, double t);
		void UpdateConcentrations(int seg_i, int mem_i, double dt, double t, double Na_sl_U_Segment, double Na_sl_L_Segment, double Ca_sl_U_Segment, double Ca_sl_L_Segment, double Ca_i_neighbor, double Ca_srs_neighbor, double Na_i_neighbor);

		double Buff_Ca_DoubleBuff(double ca_t, double buff1_tot, double buff2_tot, double Kmbuff1, double Kmbuff2);
		
		double check_State_Change(double oldval, double dval, double dt);
};